@@ -1,11 +1,11 @@
 use crate::processor::Word;
 use crate::r2_api::hex_encode;
-use crate::radius::{Radius, RadiusOption};
+use crate::radius::{Radius, RadiusOption, RunMode};
 use boolector::BV;
 use clap::{App, Arg};
-use std::time::Instant;
-use std::fs;
 use std::path::Path;
+use std::time::Instant;
+use std::{fs, process};
 
 use crate::state::StateStatus;
 use crate::value::Value;
@@ -56,6 +56,7 @@ fn main() {
                 .short("L")
                 .long("libs")
                 .takes_value(true)
+                .multiple(true)
                 .help("Load libraries from path"),
         )
         .arg(
@@ -83,6 +84,11 @@ fn main() {
                 .long("lazy")
                 .help("Evaluate symbolic PC values lazily"),
         )
+        .arg(
+            Arg::with_name("crash")
+                .long("crash")
+                .help("Execution stops on invalid memory access"),
+        )
         .arg(
             Arg::with_name("selfmodify")
                 .short("M")
@@ -216,7 +222,7 @@ fn main() {
             Arg::with_name("symbol")
                 .short("s")
                 .long("symbol")
-                .value_names(&["NAME", "BITS", "num|str"])
+                .value_names(&["NAME", "BITS"])
                 .multiple(true)
                 .help("Create a symbolic value"),
         )
@@ -275,7 +281,7 @@ fn main() {
     let debug = occurs!(matches, "verbose") || occurs!(matches, "color");
     let no_sims = occurs!(matches, "no_sims");
     let profile = occurs!(matches, "profile");
-    let fuzz    = occurs!(matches, "fuzz");
+    let fuzz = occurs!(matches, "fuzz");
     let all_sims = !no_sims && libpaths.is_empty();
 
     let plugins = occurs!(matches, "plugins")
@@ -290,6 +296,7 @@ fn main() {
         RadiusOption::Strict(occurs!(matches, "strict")),
         RadiusOption::SelfModify(occurs!(matches, "selfmodify")),
         RadiusOption::ColorOutput(occurs!(matches, "color")),
+        RadiusOption::Permissions(occurs!(matches, "crash")),
         RadiusOption::Sims(!no_sims),
         RadiusOption::SimAll(all_sims),
         RadiusOption::LoadLibs(!libpaths.is_empty()),
@@ -310,6 +317,13 @@ fn main() {
 
     let path = matches.value_of("path").unwrap_or_default();
     let dir = Path::new(matches.value_of("fuzz").unwrap_or("."));
+
+    // just a guardrail cuz the error otherwise is vv unclear
+    if path != "-" && !path.contains(":") && fs::metadata(path).is_err() {
+        println!("'{}' not found", path);
+        process::exit(1);
+    }
+
     let mut radius = Radius::new_with_options(matches.value_of("path"), &options);
 
     if !dir.exists() {
@@ -332,7 +346,7 @@ fn main() {
     }
 
     let mut state = if let Some(address) = matches.value_of("address") {
-        let addr = radius.get_address(address).unwrap();
+        let addr = radius.get_address(address).unwrap_or(0);
         if path.starts_with("frida:") {
             radius.frida_state(addr)
         } else if path.starts_with("gdb:") || path.starts_with("dbg:") {
@@ -341,7 +355,13 @@ fn main() {
             radius.call_state(addr)
         }
     } else {
-        radius.entry_state()
+        // workaround for stupid reloc.__libc_start_main issue
+        let addr = radius.get_address("main").unwrap_or(0);
+        if addr != 0 {
+            radius.call_state(addr)
+        } else {
+            radius.entry_state()
+        }
     };
 
     // collect the symbol declarations
@@ -351,10 +371,19 @@ fn main() {
     let symbols: Vec<&str> = collect!(matches, "symbol");
     for i in 0..matches.occurrences_of("symbol") as usize {
         // use get_address so hex / simple ops can be used
-        let length = radius.get_address(symbols[3 * i + 1]).unwrap_or(8) as u32;
-        let sym_value = state.symbolic_value(symbols[3 * i], length);
-        let sym_name = symbols[3 * i];
-        symbol_types.insert(sym_name, symbols[3 * i + 2]);
+        let sym_name = symbols[2 * i];
+        let mut len = symbols[2 * i + 1];
+
+        if len.ends_with("n") {
+            len = &len[..len.len() - 1];
+            symbol_types.insert(sym_name, "num");
+        } else {
+            symbol_types.insert(sym_name, "str");
+        }
+
+        let length = radius.get_address(len).unwrap_or(8) as u32;
+        let sym_value = state.symbolic_value(sym_name, length);
+        //symbol_types.insert(sym_name, symbols[3 * i + 2]);
         symbol_map.insert(sym_name, sym_value.as_bv().unwrap());
         state.context.insert(sym_name.to_owned(), vec![sym_value]);
 
@@ -405,7 +434,13 @@ fn main() {
     let cons: Vec<&str> = collect!(matches, "constrain");
     for i in 0..matches.occurrences_of("constrain") as usize {
         let bv = &symbol_map[cons[2 * i]];
-        state.constrain_bytes(bv, cons[2 * i + 1]);
+        let cons = if cons[2 * i + 1].starts_with('@') {
+            &cons[2 * i + 1][1..]
+        } else {
+            &cons[2 * i + 1]
+        };
+
+        state.constrain_bytes_bv(bv, cons);
     }
 
     // collect the ESIL hooks
@@ -517,7 +552,9 @@ fn main() {
     // get code references to strings and add them to the breakpoints
     if occurs!(matches, "break_strings") {
         // need to analyze to get string refs
-        if !analyzed { radius.analyze(3); }
+        if !analyzed {
+            radius.analyze(3);
+        }
         for string in collect!(matches, "break_strings") {
             for location in radius.r2api.search_strings(string).unwrap() {
                 bps.extend(
@@ -571,18 +608,17 @@ fn main() {
         if let Some(mut end_state) = result {
             // collect the ESIL strings to evaluate after running
             let evals: Vec<&str> = collect!(matches, "evaluate_after");
-
             for eval in evals {
                 radius.processor.parse_expression(&mut end_state, eval);
             }
-
             let solve_start = Instant::now();
 
+            println!();
             for symbol in symbol_map.keys() {
                 let val = Value::Symbolic(end_state.translate(&symbol_map[symbol]).unwrap(), 0);
 
                 if let Some(bv) = end_state.solver.eval_to_bv(&val) {
-                    let str_opt = end_state.evaluate_string(&bv);
+                    let str_opt = end_state.evaluate_string_bv(&bv);
                     let sym_type = symbol_types[symbol];
                     if sym_type == "str" && str_opt.is_some() {
                         println!("  {} : {:?}", symbol, str_opt.unwrap());
@@ -594,17 +630,21 @@ fn main() {
                     println!("  {} : no satisfiable value", symbol);
                 }
             }
+            println!();
 
             if profile {
                 println!("solve time:\t{}", solve_start.elapsed().as_micros());
             }
 
             // dump program output
+            let head = "=".repeat(37);
             if occurs!(matches, "stdout") {
-                print!("{}", end_state.dump_file_string(1).unwrap_or_default());
+                let out = end_state.dump_file_string(1).unwrap_or_default();
+                println!("{}stdout{}\n{}\n{}======{}", head, head, out, head, head);
             }
             if occurs!(matches, "stderr") {
-                print!("{}", end_state.dump_file_string(2).unwrap_or_default());
+                let out = end_state.dump_file_string(2).unwrap_or_default();
+                println!("\n{}stderr{}\n{}\n{}======{}", head, head, out, head, head);
             }
         }
     } else {
@@ -614,7 +654,7 @@ fn main() {
         states.push_back(state);
 
         let mut file_counts: HashMap<&str, usize> = HashMap::new();
-            
+
         for symbol in symbol_map.keys() {
             file_counts.insert(symbol, 0);
         }
@@ -627,29 +667,24 @@ fn main() {
             radius.processor.fetch_instruction(&mut s, cpc);
             let tn = radius.processor.instructions[&cpc].instruction.type_num;
 
-            let mut new_states = radius.processor.step(&mut s);
-            new_states.push(s);
+            let new_states = radius.processor.run(s, RunMode::Step);
+            //new_states.push(s);
 
             for mut new_state in new_states {
                 let pc = new_state.registers.get_pc().as_u64().unwrap();
                 let active = new_state.status == StateStatus::Active;
-                if pcs.contains_key(&pc) {
-                    let count = pcs[&pc];
-                    pcs.insert(pc, count + 1);
+                if pcs.entry(pc).and_modify(|c| *c += 1).or_insert(1) > &mut 1 {
                     if active && num_states <= max_states {
                         states.push_back(new_state);
                     }
                     continue;
                 }
-                pcs.insert(pc, 1);
                 // after (conditional) calls and jumps
                 if tn & 0xf == 1 || tn & 0xf == 4 {
                     for symbol in symbol_map.keys() {
-                        let val = new_state
-                            .translate(&symbol_map[symbol])
-                            .unwrap();
-                        
-                        if let Some(bytes) = new_state.evaluate_bytes(&val) {
+                        let val = new_state.translate(&symbol_map[symbol]).unwrap();
+
+                        if let Some(bytes) = new_state.evaluate_bytes_bv(&val) {
                             if !solutions.contains_key(&bytes) {
                                 let c = file_counts[symbol];
                                 let filename = &format!("{}{:04}", symbol, c);
