@@ -7,7 +7,6 @@ use crate::solver::Solver;
 use crate::value::Value;
 use std::mem;
 
-// const CHUNK: u64 = 8;
 const READ_CACHE: usize = 64;
 // const LEN_MAX: u64 = 65536;
 
@@ -17,8 +16,6 @@ const HEAP_START: u64 = 0x40000000;
 const HEAP_SIZE: u64 = 0x04000000;
 // const HEAP_CHUNK: u64 = 0x100;
 
-// pub const CHECK_PERMS: bool = false;
-
 // i think these are different on darwin
 const PROT_NONE: u64 = 0x0;
 const PROT_READ: u64 = 0x1;
@@ -29,7 +26,7 @@ const PROT_EXEC: u64 = 0x4;
 pub struct Memory {
     pub solver: Solver,
     pub r2api: R2Api,
-    
+
     // TODO refactor merge to make this private
     pub mem: HashMap<u64, Value>,
     heap: Heap,
@@ -37,7 +34,6 @@ pub struct Memory {
     pub endian: Endian,
     pub segs: Vec<MemorySegment>,
     pub blank: bool,
-    pub check: bool,
 }
 
 pub enum Permission {
@@ -58,7 +54,7 @@ pub struct MemorySegment {
 }
 
 impl Memory {
-    pub fn new(r2api: &mut R2Api, btor: Solver, blank: bool, check: bool) -> Memory {
+    pub fn new(r2api: &mut R2Api, btor: Solver, blank: bool) -> Memory {
         let segments = r2api.get_segments().unwrap();
         let mut segs = vec![];
 
@@ -76,16 +72,22 @@ impl Memory {
 
         let endian = r2api.info.bin.endian.as_str();
 
+        let bin = &r2api.info.bin;
+        let bits = if bin.arch == "arm" && bin.bits == 16 {
+            32 // says "16" for 32 bit arm cuz thumb
+        } else {
+            bin.bits
+        };
+
         Memory {
             solver: btor,
             r2api: r2api.clone(),
             mem: HashMap::new(),
             heap: Heap::new(HEAP_START, HEAP_SIZE),
-            bits: r2api.info.bin.bits,
+            bits,
             endian: Endian::from_string(endian),
             segs,
             blank,
-            check,
         }
     }
 
@@ -101,7 +103,7 @@ impl Memory {
     }
 
     pub fn free(&mut self, addr: &Value) -> Value {
-        let address = addr.as_u64().unwrap(); 
+        let address = addr.as_u64().unwrap();
         if let Some(ret) = self.heap.free(address) {
             Value::Concrete(ret, addr.get_taint())
         } else {
@@ -466,17 +468,9 @@ impl Memory {
     }
 
     pub fn read(&mut self, addr: u64, length: usize, data: &mut [Value]) {
-        //println!("length {} data len {}", length, data.());
         if length == 0 {
             return;
         }
-
-        if self.check && !self.check_permission(addr, length as u64, 'r') {
-            // everything needs to be reworked to have Result<...>
-            // so that we can properly handle things like this
-            self.handle_segfault(addr, length as u64, 'r');
-        }
-
         let make_sym = self.blank && !self.check_permission(addr, length as u64, 'i');
 
         //let mut data: Vec<Value> = Vec::with_capacity(length);
@@ -546,7 +540,7 @@ impl Memory {
 
     //read utf8 string
     pub fn read_string(&mut self, addr: u64, length: usize, solver: &mut Solver) -> String {
-        String::from_utf8(self.read_bytes(addr, length, solver)).unwrap()
+        String::from_utf8(self.read_bytes(addr, length, solver)).unwrap_or_default()
     }
 
     pub fn write_string(&mut self, addr: u64, string: &str) {
@@ -560,13 +554,7 @@ impl Memory {
     }
 
     pub fn write(&mut self, addr: u64, data: &mut [Value]) {
-        //println!("write {:?}", data);
         let length = data.len();
-
-        if self.check && !self.check_permission(addr, length as u64, 'w') {
-            self.handle_segfault(addr, length as u64, 'w');
-        }
-
         for (count, mut item) in data.iter_mut().enumerate().take(length) {
             let caddr = addr + count as u64;
             self.mem.insert(caddr, mem::take(&mut item));
