@@ -79,7 +79,8 @@ pub enum Operations {
     GoTo,
     ToDo,
     NoOperation,
-    Print, // Tool for cli hooks
+    Print,      // Tool for cli hooks
+    PrintDebug, // Tool for cli hooks
     Constrain,
     Terminate,
 
@@ -183,6 +184,7 @@ impl Operations {
 
             // hax for use in the cli / plugin
             "." => Operations::Print,
+            ".." => Operations::PrintDebug,
             "_" => Operations::Constrain,
             "!!" => Operations::Terminate,
 
@@ -631,7 +633,7 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
             state.esil.previous = addr;
             state.esil.last_sz = 8 * n;
         }
-        // this is a shit hack to do op pokes ~efficiently
+        // this is a hack to do op pokes ~efficiently
         Operations::AddressStore => {
             let addr = pop_value(state, false, false);
             state.esil.stored_address = Some(addr.to_owned());
@@ -798,18 +800,24 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
             let value = pop_value(state, false, false);
             let ip = state.registers.get_pc().as_u64().unwrap_or_default();
             if let Some(bv) = state.solver.eval_to_bv(&value) {
-                if let Some(string) = state.evaluate_string(&bv) {
-                    println!("{:016x}: {:?} {:?}", ip, bv, string);
+                let hex = &format!("{:?}", bv)[2..];
+                if let Some(string) = state.evaluate_string_bv(&bv) {
+                    println!("\n{:016x}: 0x{} {:?}\n", ip, hex, string);
                 } else {
-                    println!("{:016x}: {:?}", ip, bv);
+                    println!("\n{:016x}: 0x{}\n", ip, hex);
                 }
             } else {
-                println!("{:016x}: unsat", ip);
+                println!("\n{:016x}: unsat\n", ip);
             }
         }
+        Operations::PrintDebug => {
+            let value = pop_value(state, false, false);
+            let ip = state.registers.get_pc().as_u64().unwrap_or_default();
+            println!("\n{:016x}: {:?}\n", ip, value);
+        }
         Operations::Constrain => {
             let value = pop_value(state, false, false);
-            state.assert_value(&value);
+            state.assert(&value);
         }
         Operations::Terminate => {
             state.set_break();
@@ -858,6 +866,7 @@ pub fn do_operation(state: &mut State, operation: &Operations) {
                 let cur = state.esil.current.to_owned();
                 let lsb = cur & Value::Concrete(0xff, 0);
                 let pf = !((((lsb * c1) & c2) % c3) & Value::Concrete(1, 0));
+                //let pf = Value::Symbolic(val.redxor(), *t); this is 2x slower wtf
                 push_value(state, pf);
             }
         },
