@@ -2,7 +2,7 @@ use crate::operations::{
     do_operation, pop_concrete, pop_stack_value, pop_value, push_value, Operations, OPS,
 };
 use crate::r2_api::{hex_decode, CallingConvention, Instruction, Syscall};
-use crate::value::Value;
+use crate::value::{vc, Value};
 
 use crate::state::{
     Event, EventContext, EventTrigger, ExecMode, StackItem, State, StateStatus, DO_EVENT_HOOKS,
@@ -11,10 +11,13 @@ use crate::state::{
 use crate::sims::syscall::syscall;
 use crate::sims::SimMethod;
 
-use ahash::{AHashMap, AHashSet};
-use std::collections::VecDeque;
+use std::collections::BinaryHeap;
 use std::mem;
+use std::rc::Rc;
+
+use ahash::{AHashMap, AHashSet};
 type HashMap<P, Q> = AHashMap<P, Q>;
+use std::collections::BTreeMap;
 
 const INSTR_NUM: usize = 64;
 // const COLOR: bool = true;
@@ -30,12 +33,20 @@ pub enum Word {
     Unknown(String),
 }
 
-pub type HookMethod = fn(&mut State) -> bool;
+#[derive(Debug, Clone, PartialEq)]
+pub enum RunMode {
+    Single,
+    Step,
+    Parallel,
+    Multiple,
+}
+
+pub type HookMethod = dyn Fn(&mut State) -> bool;
 
 #[derive(Clone)]
 pub struct Processor {
-    pub instructions: HashMap<u64, InstructionEntry>,
-    pub hooks: HashMap<u64, Vec<HookMethod>>,
+    pub instructions: BTreeMap<u64, InstructionEntry>,
+    pub hooks: HashMap<u64, Vec<Rc<HookMethod>>>,
     pub esil_hooks: HashMap<u64, Vec<String>>,
     pub sims: HashMap<u64, SimMethod>,
     pub traps: HashMap<u64, SimMethod>,
@@ -45,6 +56,7 @@ pub struct Processor {
     pub avoidpoints: AHashSet<u64>,
     pub visited: AHashSet<u64>,
     pub merges: HashMap<u64, State>,
+    pub crashes: Vec<State>,
     pub selfmodify: bool,
     pub optimized: bool,
     pub debug: bool,
@@ -55,9 +67,9 @@ pub struct Processor {
     pub steps: u64,        // number of state steps
 }
 
-#[derive(Debug, Clone, PartialEq)]
-pub enum InstructionStatus {
-    None,
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+pub enum InstructionFlag {
+    //None,
     Hook,
     ESILHook,
     Sim,
@@ -70,7 +82,7 @@ pub enum InstructionStatus {
 pub struct InstructionEntry {
     pub instruction: Instruction,
     pub tokens: Vec<Word>,
-    pub status: InstructionStatus, // next: Option<Arc<InstructionEntry>>
+    pub flags: AHashSet<InstructionFlag>, // next: Option<Arc<InstructionEntry>>
 }
 
 //const DEBUG: bool = false; // show instructions
@@ -88,10 +100,10 @@ impl Processor {
         lazy: bool,
         force: bool,
         topological: bool,
-        color: bool
+        color: bool,
     ) -> Self {
         Processor {
-            instructions: HashMap::new(),
+            instructions: BTreeMap::new(),
             hooks: HashMap::new(),
             esil_hooks: HashMap::new(),
             sims: HashMap::new(),
@@ -102,6 +114,7 @@ impl Processor {
             avoidpoints: AHashSet::new(),
             visited: AHashSet::new(),
             merges: HashMap::new(),
+            crashes: vec![],
             selfmodify,
             optimized,
             debug,
@@ -160,6 +173,7 @@ impl Processor {
     }
 
     /// attempt to tokenize word as number literal (eg. 0x8)
+    #[inline]
     pub fn get_literal(&self, word: &str) -> Option<Word> {
         if let Ok(i) = word.parse::<u64>() {
             Some(Word::Literal(Value::Concrete(i, 0)))
@@ -174,6 +188,7 @@ impl Processor {
     }
 
     /// attempt to tokenize word as register (eg. rbx)
+    #[inline]
     pub fn get_register(&self, state: &mut State, word: &str) -> Option<Word> {
         let name = if let Some(alias) = state.registers.aliases.get(word) {
             alias.reg.as_str()
@@ -187,6 +202,7 @@ impl Processor {
     }
 
     /// attempt to tokenize word as operation (eg. +)
+    #[inline]
     pub fn get_operator(&self, word: &str) -> Option<Word> {
         match Operations::from_string(word) {
             Operations::Unknown => None,
@@ -511,11 +527,11 @@ impl Processor {
         &self,
         state: &mut State,
         instr: &Instruction,
-        status: &InstructionStatus,
+        flags: &AHashSet<InstructionFlag>,
         words: &[Word],
     ) {
-        if state.memory.check && !state.memory.check_permission(instr.offset, instr.size, 'x') {
-            state.memory.handle_segfault(instr.offset, instr.size, 'x');
+        if state.check && state.check_crash(&vc(instr.offset), &vc(instr.size), 'x') {
+            return;
         }
 
         let pc = instr.offset;
@@ -530,25 +546,23 @@ impl Processor {
             }
         }
 
-        // shit is gettin messy
-        let mut new_status = status;
-        if state.status == StateStatus::PostMerge && *status == InstructionStatus::Merge {
+        //let mut new_status = status;
+        let mut new_flags = flags.clone();
+        if state.status == StateStatus::PostMerge && flags.contains(&InstructionFlag::Merge) {
             state.status = StateStatus::Active;
-            new_status = &InstructionStatus::None;
+            new_flags.remove(&InstructionFlag::Merge);
         }
 
         match instr.type_num {
             CALL_TYPE => state.backtrace.push((instr.jump as u64, new_pc)),
             RETN_TYPE => {
-                if state.backtrace.is_empty() && new_status == &InstructionStatus::None {
+                if state.backtrace.is_empty() && new_flags.is_empty() {
                     // try to avoid returning outside valid context
-                    if !self.breakpoints.is_empty() 
-                        || !self.esil_hooks.is_empty() {
-
-                        new_status = &InstructionStatus::Avoid;
+                    if !self.breakpoints.is_empty() || !self.esil_hooks.is_empty() {
+                        new_flags.insert(InstructionFlag::Avoid);
                     } else {
                         // break if there are no other breakpoints/hooks
-                        new_status = &InstructionStatus::Break;
+                        new_flags.insert(InstructionFlag::Break);
                     }
                 } else {
                     state.backtrace.pop();
@@ -557,67 +571,69 @@ impl Processor {
             _ => {}
         }
 
-        match new_status {
-            InstructionStatus::None => {
-                let pc_val = Value::Concrete(new_pc, 0);
-                state.registers.set_pc(pc_val);
-                self.parse(state, words);
-            }
-            InstructionStatus::Hook => {
-                let mut skip = false;
-                let pc_val = Value::Concrete(new_pc, 0);
-                state.registers.set_pc(pc_val);
-
+        // skip executing this instruction
+        let mut skip = false;
+        let mut update = true;
+        if !new_flags.is_empty() {
+            if new_flags.contains(&InstructionFlag::Hook) {
                 let hooks = &self.hooks[&pc];
                 for hook in hooks {
                     skip = !hook(state) || skip;
                 }
-
-                if !skip {
-                    self.parse(state, words);
-                }
             }
-            InstructionStatus::ESILHook => {
-                let mut skip = false;
-                let pc_val = Value::Concrete(new_pc, 0);
-                state.registers.set_pc(pc_val);
-
+            if new_flags.contains(&InstructionFlag::ESILHook) {
                 let esils = &self.esil_hooks[&pc];
                 for esil in esils {
                     self.parse_expression(state, esil);
                     let val = pop_concrete(state, false, false);
                     skip = (val != 0) || skip;
                 }
-
-                if !skip {
-                    self.parse(state, words);
-                }
             }
-            InstructionStatus::Sim => {
+            if state.registers.get_pc() != vc(pc) {
+                update = false; // hook changed pc dont update
+            }
+            if new_flags.contains(&InstructionFlag::Sim) {
                 let sim = &self.sims[&pc];
                 let cc = state.r2api.get_cc(pc).unwrap_or_default();
                 let args = self.get_args(state, &cc);
 
-                let pc_val = Value::Concrete(new_pc, 0);
-                state.registers.set_pc(pc_val);
-
                 let ret = sim(state, &args);
                 state.registers.set_with_alias(cc.ret.as_str(), ret);
                 state.backtrace.pop();
 
                 // don't ret if sim changes the PC value
                 // this is bad hax because thats all i do
-                let newer_pc_val = state.registers.get_pc();
-                if let Some(newer_pc) = newer_pc_val.as_u64() {
-                    if newer_pc == new_pc {
-                        self.ret(state);
-                    }
+                if state.registers.get_pc() == vc(pc) {
+                    self.ret(state);
                 }
+                skip = true;
+                update = false;
             }
-            InstructionStatus::Break => state.status = StateStatus::Break,
-            InstructionStatus::Merge => state.status = StateStatus::Merge,
-            InstructionStatus::Avoid => state.status = StateStatus::Inactive,
-        };
+            if new_flags.contains(&InstructionFlag::Break) {
+                state.status = StateStatus::Break;
+                skip = true;
+                update = false;
+            }
+            if new_flags.contains(&InstructionFlag::Merge) {
+                state.status = StateStatus::Merge;
+                skip = true;
+                update = false;
+            }
+            if new_flags.contains(&InstructionFlag::Avoid) {
+                state.status = StateStatus::Inactive;
+                skip = true;
+                update = false;
+            }
+        }
+
+        if update {
+            let pc_val = Value::Concrete(new_pc, 0);
+            state.registers.set_pc(pc_val);
+        }
+
+        if !skip {
+            self.parse(state, words);
+        }
     }
 
     // weird method that just performs a return
@@ -655,31 +671,36 @@ impl Processor {
                 let size = instr.size;
                 let words = self.tokenize(state, &instr.esil);
 
-                let mut status = InstructionStatus::None;
+                let mut flags = AHashSet::new();
                 let mut opt = self.optimized && !self.selfmodify;
                 if self.hooks.contains_key(&pc_tmp) {
-                    status = InstructionStatus::Hook;
-                } else if self.esil_hooks.contains_key(&pc_tmp) {
-                    status = InstructionStatus::ESILHook;
-                } else if self.breakpoints.contains(&pc_tmp) {
-                    status = InstructionStatus::Break;
-                } else if self.mergepoints.contains(&pc_tmp) {
-                    status = InstructionStatus::Merge;
-                } else if self.avoidpoints.contains(&pc_tmp) {
-                    status = InstructionStatus::Avoid;
-                } else if self.sims.contains_key(&pc_tmp) {
-                    status = InstructionStatus::Sim;
+                    flags.insert(InstructionFlag::Hook);
+                }
+                if self.esil_hooks.contains_key(&pc_tmp) {
+                    flags.insert(InstructionFlag::ESILHook);
+                }
+                if self.breakpoints.contains(&pc_tmp) {
+                    flags.insert(InstructionFlag::Break);
+                }
+                if self.mergepoints.contains(&pc_tmp) {
+                    flags.insert(InstructionFlag::Merge);
+                }
+                if self.avoidpoints.contains(&pc_tmp) {
+                    flags.insert(InstructionFlag::Avoid);
+                }
+                if self.sims.contains_key(&pc_tmp) {
+                    flags.insert(InstructionFlag::Sim);
                 }
 
                 // don't optimize if hooked / bp for accuracy
-                if status != InstructionStatus::None {
+                if !flags.is_empty() {
                     opt = false;
                 }
 
                 let instr_entry = InstructionEntry {
                     instruction: instr,
                     tokens: words,
-                    status,
+                    flags,
                 };
 
                 if opt {
@@ -707,12 +728,13 @@ impl Processor {
             panic!("Executed invalid instruction");
         }
 
-        self.execute(state, &instr.instruction, &instr.status, &instr.tokens);
+        self.execute(state, &instr.instruction, &instr.flags, &instr.tokens);
     }
 
     /// Take single step with the state provided
     pub fn step(&mut self, state: &mut State) -> Vec<State> {
         self.steps += 1;
+        state.visit();
 
         let pc_allocs = 32;
         let pc_value = state.registers.get_pc();
@@ -760,10 +782,7 @@ impl Processor {
             if DO_EVENT_HOOKS && state.has_event_hooks {
                 state.do_hooked(
                     &Event::SymbolicExec(EventTrigger::After),
-                    &EventContext::ExecContext(
-                        new_pc.clone(), 
-                        state.esil.pcs.clone()
-                    ),
+                    &EventContext::ExecContext(new_pc.clone(), state.esil.pcs.clone()),
                 );
             }
         }
@@ -776,7 +795,7 @@ impl Processor {
                 let mut new_state = state.clone();
                 if let Some(pc_val) = new_pc.as_bv() {
                     let a = pc_val._eq(&new_state.bvv(*new_pc_val, pc_val.get_width()));
-                    new_state.solver.assert(&a);
+                    new_state.solver.assert_bv(&a);
                 }
                 new_state.registers.set_pc(Value::Concrete(*new_pc_val, 0));
                 states.push(new_state);
@@ -786,7 +805,7 @@ impl Processor {
             if let Some(pc_val) = new_pc.as_bv() {
                 let pc_bv = pc_val;
                 let a = pc_bv._eq(&state.bvv(new_pc_val, pc_bv.get_width()));
-                state.solver.assert(&a);
+                state.solver.assert_bv(&a);
             }
             state.registers.set_pc(Value::Concrete(new_pc_val, 0));
             states
@@ -795,95 +814,85 @@ impl Processor {
         }
     }
 
-    /// run the state until a breakpoint is hit or state split
-    pub fn run(&mut self, state: State, split: bool) -> VecDeque<State> {
-        let mut states = VecDeque::with_capacity(state.solver.eval_max);
-        states.push_back(state);
+    /// run the state until completion based on mode
+    pub fn run(&mut self, state: State, mode: RunMode) -> Vec<State> {
+        // use binary heap as priority queue to prioritize states
+        // that have the lowest number of visits for the current PC
+        let mut states = BinaryHeap::new();
+        let mut results = vec![];
+        states.push(Rc::new(state));
 
-        // run until empty for single threaded, until split for multi
-        let mut index = 0;
-        while !split || (states.len() == 1) {
+        // run until empty for single, until split for parallel
+        // or until every state is at the breakpoint for multiple
+        let split = mode == RunMode::Parallel;
+        let step = mode == RunMode::Step;
+
+        loop {
             if states.is_empty() {
                 if self.merges.is_empty() {
-                    return VecDeque::new();
+                    return results;
                 } else {
                     // pop one out of mergers
                     let key = *self.merges.keys().next().unwrap();
                     let mut merge = self.merges.remove(&key).unwrap();
                     merge.status = StateStatus::PostMerge;
-                    states.push_back(merge)
+                    states.push(Rc::new(merge));
                 }
             }
 
-            let current_state = &mut states[index];
+            let mut current_rc = states.pop().unwrap();
+            let current_state = Rc::make_mut(&mut current_rc);
 
             match current_state.status {
                 StateStatus::Active | StateStatus::PostMerge => {
                     let new_states = self.step(current_state);
-
-                    let pc = current_state
-                        .registers
-                        .get_pc()
-                        .as_u64()
-                        .unwrap_or_default();
-
-                    for s in new_states {
-                        states.push_front(s);
-                        index += 1;
-                    }
-                    
-                    // if the current address has not been
-                    // seen before, keep executing it 
-                    if self.visited.contains(&pc) {
-                        index += 1;
-                    } else {
-                        self.visited.insert(pc);
+                    for state in new_states {
+                        states.push(Rc::new(state));
                     }
+                    states.push(current_rc);
                 }
                 StateStatus::Merge => {
-                    self.merge(states.remove(index).unwrap());
+                    self.merge(current_state.to_owned());
                 }
                 StateStatus::Break => {
                     if current_state.is_sat() {
-                        return VecDeque::from(vec![states.remove(index).unwrap()]);
-                    } else {
-                        states.remove(index).unwrap();
+                        results.push(current_state.to_owned());
+                        if mode != RunMode::Multiple {
+                            return results;
+                        }
                     }
                 }
-                _ => {
-                    states.remove(index).unwrap();
+                StateStatus::Crash(_addr, _len) => {
+                    self.crashes.push(current_state.to_owned());
                 }
+                _ => {}
             }
-            index = if states.is_empty() {
-                0
-            } else {
-                index % states.len()
-            };
-        }
 
-        states
+            // single step mode always returns states
+            if step || (split && states.len() > 1) {
+                while let Some(mut state) = states.pop() {
+                    results.push(Rc::make_mut(&mut state).to_owned());
+                }
+                return results;
+            }
+        }
     }
 
     pub fn merge(&mut self, mut state: State) {
-        let pc = state.registers.get_with_alias("PC").as_u64().unwrap();
+        let pc = state.registers.get_pc().as_u64().unwrap();
 
-        let has_pc = self.merges.contains_key(&pc);
-        if !has_pc {
-            // trick clippy idk
-            self.merges.insert(pc, state);
-        } else {
-            let mut merge_state = self.merges.remove(&pc).unwrap();
-            let state_asserts = state.solver.assertions.clone();
+        if let Some(mut merge_state) = self.merges.remove(&pc) {
+            let state_asserts = &state.solver.assertions;
             let assertion = state.solver.and_all(&state_asserts);
-            let asserted = Value::Symbolic(assertion.clone(), 0);
+            let asserted = Value::Symbolic(assertion, 0);
 
             // merge registers
             let mut new_regs = Vec::with_capacity(256);
             let reg_count = state.registers.values.len();
             for index in 0..reg_count {
                 let reg = &merge_state.registers.values[index];
-                let curr_reg = state.registers.values[index].clone();
-                new_regs.push(state.solver.conditional(&asserted, &curr_reg, &reg));
+                let curr_reg = &state.registers.values[index];
+                new_regs.push(state.solver.conditional(&asserted, curr_reg, reg));
             }
             merge_state.registers.values = new_regs;
 
@@ -903,11 +912,13 @@ impl Processor {
             merge_state.memory.mem = new_mem;
 
             // merge solvers
-            let assertions = merge_state.solver.assertions.clone();
-            let current = state.solver.and_all(&assertions);
+            let assertions = &merge_state.solver.assertions;
+            let current = state.solver.and_all(assertions);
             merge_state.solver.reset();
-            merge_state.assert(&current.or(&assertion));
+            merge_state.assert_bv(&current.or(&asserted.as_bv().unwrap()));
             self.merges.insert(pc, merge_state);
+        } else {
+            self.merges.insert(pc, state);
         }
     }
 }
