@@ -421,6 +421,8 @@ pub struct R2Api {
     //pub permissions: HashMap<u64, Permission>,
     pub info: Information,
     pub mode: Mode,
+    do_cache: bool,
+    cache: HashMap<String, String>,
 }
 
 impl R2Api {
@@ -440,6 +442,8 @@ impl R2Api {
             r2p: Arc::new(Mutex::new(r2pipe.unwrap())),
             info: Information::default(),
             mode: Mode::Default,
+            do_cache: false,
+            cache: HashMap::new(),
         };
 
         r2api.info = r2api.get_info().unwrap();
@@ -464,6 +468,21 @@ impl R2Api {
         Ok(self.r2p.lock().unwrap().cmd(cmd).unwrap_or_default())
     }
 
+    // cached command
+    pub fn ccmd(&mut self, cmd: &str) -> R2Result<String> {
+        if self.do_cache {
+            if let Some(result) = self.cache.get(cmd) {
+                Ok(result.to_owned())
+            } else {
+                let result = self.cmd(cmd)?;
+                self.cache.insert(cmd.to_owned(), result.clone());
+                Ok(result)
+            }
+        } else {
+            self.cmd(cmd)
+        }
+    }
+
     pub fn get_info(&mut self) -> R2Result<Information> {
         let json = self.cmd("ij")?;
         Ok(serde_json::from_str(json.as_str()).unwrap())
@@ -515,7 +534,7 @@ impl R2Api {
     */
 
     pub fn get_syscall_cc(&mut self) -> R2Result<CallingConvention> {
-        // this sucks, need a central place for arch shit
+        // this sucks, need a central place for arch stuff
         match (self.info.bin.arch.as_str(), self.info.bin.bits) {
             ("x86", 32) => Ok(CallingConvention {
                 args: vec![
@@ -811,6 +830,7 @@ impl R2Api {
         data: &[u8],
         num: usize,
     ) -> R2Result<Vec<Instruction>> {
+        // this is unfortunately necessary as there is no padj @, i need to make one
         let cmd = format!("wx {} @ {}; pij {} @ {}", hex_encode(data), addr, num, addr);
 
         let json = self.cmd(cmd.as_str())?;
@@ -842,7 +862,11 @@ impl R2Api {
         };
 
         let val = self.cmd(cmd.as_str())?;
-        r2_result(u64::from_str_radix(&val[2..val.len() - 1], 16))
+        if val.len() > 3 {
+            r2_result(u64::from_str_radix(&val[2..val.len() - 1], 16))
+        } else {
+            Ok(0) // to be consistent with r2?
+        }
     }
 
     pub fn get_files(&mut self) -> R2Result<Vec<File>> {
@@ -913,7 +937,7 @@ impl R2Api {
                     .to_str()
                     .unwrap_or_default()
                     .to_owned();
-                    
+
                 let loaded = full_paths.iter().any(|x| x == &lib_path);
                 if !loaded && Path::new(&lib_path).exists() {
                     let load_addr = (high_addr & 0xfffffffffffff000) + 0x3000; // idk
