@@ -1,14 +1,16 @@
-use crate::processor::{HookMethod, Processor};
+pub use crate::processor::{HookMethod, Processor, RunMode};
 use crate::r2_api::{BasicBlock, FunctionInfo, Information, Instruction, R2Api, R2Result};
-use crate::state::{State, StateStatus};
+use crate::state::State;
 //use crate::value::Value;
 use crate::sims::syscall::indirect;
 use crate::sims::{get_sims, zero, SimMethod};
-use crate::value::Value;
+use crate::value::{vc, Value};
 
-use std::collections::VecDeque;
+// use std::collections::VecDeque;
+use std::rc::Rc;
 use std::sync::{Arc, Mutex};
-use std::thread;
+
+// use std::thread;
 
 use ahash::AHashMap;
 type HashMap<P, Q> = AHashMap<P, Q>;
@@ -21,7 +23,7 @@ pub enum RadiusOption {
     Sims(bool),
     /// Sim all imports, with stub if missing        
     SimAll(bool),
-    /// Optimize executed ESIL expressions      
+    /// Optimize executed ESIL expressions
     Optimize(bool),
     /// Enable debug output
     Debug(bool),
@@ -118,13 +120,14 @@ impl Radius {
         }
 
         let debug = options.contains(&RadiusOption::Debug(true));
+        let color = options.contains(&RadiusOption::ColorOutput(true));
         let use_sims = !options.contains(&RadiusOption::Sims(false));
 
         if !options.contains(&RadiusOption::LoadPlugins(true)) {
             argv.push("-NN");
         }
 
-        if debug {
+        if debug && color {
             // pretty print disasm + esil
             argv.push("-e scr.color=3");
             argv.push("-e asm.cmt.esil=true");
@@ -138,10 +141,7 @@ impl Radius {
 
         let mut r2api = R2Api::new(filename, args);
         r2api.set_option("io.cache", "true").unwrap();
-        r2api.cmd("eco darkda").unwrap(); // i like darkda
-
-        // doesnt work with io.cache? and not needed?
-        // r2api.set_option("bin.cache", "true").unwrap();
+        // r2api.cmd("eco darkda").unwrap(); // i like darkda
 
         let arch = &r2api.info.bin.arch;
 
@@ -157,7 +157,6 @@ impl Radius {
         let sim_all = options.contains(&RadiusOption::SimAll(true));
         let selfmod = options.contains(&RadiusOption::SelfModify(true));
         let strict = options.contains(&RadiusOption::Strict(true));
-        let color = options.contains(&RadiusOption::ColorOutput(true));
 
         let mut processor = Processor::new(selfmod, opt, debug, lazy, force, topo, color);
         let processors = Arc::new(Mutex::new(vec![]));
@@ -233,10 +232,9 @@ impl Radius {
         let context = self.r2api.init_frida(addr).unwrap();
 
         for reg in context.keys() {
-            let val = u64::from_str_radix(&context[reg][2..], 16).unwrap();
+            let val = u64::from_str_radix(&context[reg][2..], 16).unwrap_or(0);
             state.registers.set(reg, vc(val));
         }
-
         state
     }
 
@@ -263,7 +261,7 @@ impl Radius {
 
         let start_main_reloc = self.r2api.get_address("reloc.__libc_start_main").unwrap();
 
-        self.hook(start_main_reloc, __libc_start_main);
+        self.hook(start_main_reloc, Rc::new(__libc_start_main));
         state
     }
 
@@ -349,21 +347,25 @@ impl Radius {
     }
 
     /// Hook an address with a callback that is passed the `State`
-    pub fn hook(&mut self, addr: u64, hook_callback: HookMethod) {
-        let mut hooks = self.processor.hooks.remove(&addr).unwrap_or_default();
-        hooks.push(hook_callback);
-        self.processor.hooks.insert(addr, hooks);
+    pub fn hook(&mut self, addr: u64, hook_callback: Rc<HookMethod>) {
+        self.processor
+            .hooks
+            .entry(addr)
+            .or_insert(vec![])
+            .push(hook_callback);
     }
 
     /// Hook an address with an esil expression
     pub fn esil_hook(&mut self, addr: u64, esil: &str) {
-        let mut esils = self.processor.esil_hooks.remove(&addr).unwrap_or_default();
-        esils.push(esil.to_owned());
-        self.processor.esil_hooks.insert(addr, esils);
+        self.processor
+            .esil_hooks
+            .entry(addr)
+            .or_insert(vec![])
+            .push(esil.to_owned());
     }
 
     /// Hook a symbol with a callback that is passed each state that reaches it
-    pub fn hook_symbol(&mut self, sym: &str, hook_callback: HookMethod) {
+    pub fn hook_symbol(&mut self, sym: &str, hook_callback: Rc<HookMethod>) {
         let addr = self.get_address(sym).unwrap();
         self.hook(addr, hook_callback);
     }
@@ -447,7 +449,12 @@ impl Radius {
     pub fn run_until(&mut self, state: State, target: u64, avoid: &[u64]) -> Option<State> {
         self.breakpoint(target);
         self.avoid(avoid);
-        self.processor.run(state, false).pop_front()
+        self.processor.run(state, RunMode::Single).pop()
+    }
+
+    /// execute until every state has reached an end
+    pub fn run_all(&mut self, state: State) -> Vec<State> {
+        self.processor.run(state, RunMode::Multiple)
     }
 
     /**
@@ -456,21 +463,25 @@ impl Radius {
      * More words
      *
      */
-    pub fn run(&mut self, state: State, threads: usize) -> Option<State> {
+    pub fn run(&mut self, state: State, _threads: usize) -> Option<State> {
+        // we are gonna scrap threads for now cuz theyre currently useless.
+        self.processor.run(state, RunMode::Single).pop()
+
         // if there are multiple threads we need to duplicate solvers
         // else there will be race conditions. Unfortunately this
         // will prevent mergers from happening. this sucks
-        if threads == 1 || !self.processor.mergepoints.is_empty() {
-            return self.processor.run(state, false).pop_front();
+
+        /*if threads == 1 || !self.processor.mergepoints.is_empty() {
+            return self.processor.run(state, RunMode::Single).pop();
         }
 
         let mut handles = Vec::with_capacity(threads);
-        let statevector = Arc::new(Mutex::new(VecDeque::with_capacity(threads*self.eval_max)));
+        let statevector = Arc::new(Mutex::new(VecDeque::with_capacity(threads * self.eval_max)));
         statevector.lock().unwrap().push_back(state);
 
         loop {
             let mut count = 0;
-            let state_count = statevector.lock().unwrap().len();
+            let _state_count = statevector.lock().unwrap().len();
             while count < threads && !statevector.lock().unwrap().is_empty() {
                 //println!("on thread {}!", thread_count);
                 let procs = self.processors.clone();
@@ -484,7 +495,7 @@ impl Radius {
                 };
 
                 let handle = thread::spawn(move || {
-                    let new_states = processor.run(state, state_count < threads);
+                    let new_states = processor.run(state, RunMode::Single);
                     states.lock().unwrap().extend(new_states);
                     procs.lock().unwrap().push(processor);
                 });
@@ -508,7 +519,7 @@ impl Radius {
             {
                 break Some(state.to_owned());
             }
-        }
+        }*/
     }
 
     /// Run radare2 analysis
@@ -603,8 +614,3 @@ pub fn __libc_start_main(state: &mut State) -> bool {
 
     false
 }
-
-/// convenience method for making an untainted `Value::Concrete`
-pub fn vc(v: u64) -> Value {
-    Value::Concrete(v, 0)
-}
