@@ -1,24 +1,24 @@
+use crate::sims::format;
 use crate::sims::syscall;
 use crate::state::State;
-use crate::value::Value;
+use crate::value::{vc, Value};
 use rand::Rng;
 
 const MAX_LEN: u64 = 8192;
 
 // TODO everything that interacts with errno in any way
-// I forget how errno works, i know its weird
 
 // now using sim fs
 pub fn puts(state: &mut State, args: &[Value]) -> Value {
-    put_help(state, args, true)
+    put_helper(state, args, true)
 }
 
-pub fn put_help(state: &mut State, args: &[Value], nl: bool) -> Value {
+fn put_helper(state: &mut State, args: &[Value], nl: bool) -> Value {
     let addr = &args[0];
     let length = strlen(state, &args[0..1]);
     let mut data = state.memory_read(addr, &length);
     if nl {
-        data.push(Value::Concrete('\n' as u64, 0));
+        data.push(vc('\n' as u64))
     } // add newline
     state.filesystem.write(1, data);
     length
@@ -30,6 +30,36 @@ pub fn putchar(state: &mut State, args: &[Value]) -> Value {
     c
 }
 
+fn readline(state: &mut State, args: &[Value]) -> Value {
+    let fd = state.solver.evalcon_to_u64(&args[0]).unwrap_or(0) as usize;
+    let mut p = args[1].to_owned();
+
+    loop {
+        let c = state
+            .filesystem
+            .read(fd, 1)
+            .get(0)
+            .unwrap_or(&vc(-1i64 as u64))
+            .to_owned();
+
+        if c.as_u64() != Some(-1i64 as u64) {
+            // uhhh idk we cant do symbolic file pos yet so
+            // this is where we are at
+            if c.as_u64() == Some('\n' as u64) {
+                break;
+            } else if c.is_symbolic() {
+                state.assert(&!c.eq(&vc('\n' as u64)));
+            }
+            state.memory_write_value(&p, &c, 1);
+            p = p + vc(1);
+        } else {
+            break;
+        }
+    }
+    state.memory_write_value(&p, &vc(0), 1);
+    args[1].to_owned()
+}
+
 pub fn getchar(state: &mut State, _args: &[Value]) -> Value {
     state
         .filesystem
@@ -39,56 +69,39 @@ pub fn getchar(state: &mut State, _args: &[Value]) -> Value {
         .to_owned()
 }
 
-// TODO you know, all this
 pub fn fprintf(state: &mut State, args: &[Value]) -> Value {
-    printf(state, &args[1..])
+    let fd = fileno(state, &args[0..1]);
+    let fdn = state.solver.evalcon_to_u64(&fd).unwrap_or(1);
+    let formatted = format::format(state, args);
+    let ret = vc(formatted.len() as u64);
+    state.filesystem.write(fdn as usize, formatted);
+    ret
+}
+
+pub fn sprintf(state: &mut State, args: &[Value]) -> Value {
+    let formatted = format::format(state, &args[1..]);
+    let ret = vc(formatted.len() as u64);
+    state.memory_write(&args[0], &formatted, &vc(formatted.len() as u64));
+    ret
 }
 
-// TODO you know, all this
 pub fn printf(state: &mut State, args: &[Value]) -> Value {
-    let length_val = state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0));
-    let length = state.solver.eval_to_u64(&length_val).unwrap();
-    let addr = state.solver.eval_to_u64(&args[0]).unwrap();
-    let format = state.memory_read_string(addr, length as usize);
-    if &format == "%s" || &format == "%s\n" {
-        put_help(state, &args[1..2], &format != "%s")
-    } else if &format == "%d" || &format == "%d\n" {
-        let addr = state.memory_alloc(&vc(64));
-        itoa_helper(state, &args[1], &addr, &vc(10), true, 32);
-        put_help(state, &[addr], &format != "%d")
-    } else if &format == "%x" || &format == "%x\n" {
-        let addr = state.memory_alloc(&vc(64));
-        itoa_helper(state, &args[1], &addr, &vc(16), true, 32);
-        put_help(state, &[addr], &format != "%x")
-    } else {
-        put_help(state, &args[0..1], false)
-    }
+    let formatted = format::format(state, args);
+    let ret = vc(formatted.len() as u64);
+    state.filesystem.write(1, formatted);
+    ret
 }
 
-// TODO you know, all this
 pub fn scanf(state: &mut State, args: &[Value]) -> Value {
-    let length_val = state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0));
-    let length = state.solver.eval_to_u64(&length_val).unwrap();
-    let addr = state.solver.eval_to_u64(&args[0]).unwrap();
-    let format = state.memory_read_string(addr, length as usize);
-    if &format == "%s" {
-        gets(state, &args[1..])
-    } else if format.starts_with('%') && format.ends_with('s') {
-        if let Ok(n) = &format[1..format.len() - 1].parse::<u64>() {
-            read(state, &[vc(0), args[1].to_owned(), vc(*n)])
-        } else {
-            // uh idk
-            vc(0)
-        }
-    } else if &format == "%d" {
-        gets(state, &args[1..]);
-        atoi_helper(state, &args[1], &vc(10), 32)
-    } else if &format == "%x" {
-        gets(state, &args[1..]);
-        atoi_helper(state, &args[1], &vc(16), 32)
-    } else {
-        vc(0) // you're on your own
-    }
+    let buf = state.memory_alloc(&vc(MAX_LEN));
+    gets(state, &[buf.clone()]);
+    let result = format::scan(state, &[&[buf.clone()], args].concat());
+    state.memory_free(&buf);
+    result
+}
+
+pub fn sscanf(state: &mut State, args: &[Value]) -> Value {
+    format::scan(state, args)
 }
 
 pub fn memmove(state: &mut State, args: &[Value]) -> Value {
@@ -105,20 +118,12 @@ pub fn memcpy(state: &mut State, args: &[Value]) -> Value {
 
 pub fn bcopy(state: &mut State, args: &[Value]) -> Value {
     state.memory_move(&args[0], &args[1], &args[2]);
-    Value::Concrete(0, 0)
+    vc(0)
 }
 
 pub fn bzero(state: &mut State, args: &[Value]) -> Value {
-    memset(
-        state,
-        &[
-            args[0].to_owned(),
-            Value::Concrete(0, 0),
-            args[1].to_owned(),
-        ],
-    );
-
-    Value::Concrete(0, 0)
+    memset(state, &[args[0].to_owned(), vc(0), args[1].to_owned()]);
+    vc(0)
 }
 
 pub fn mempcpy(state: &mut State, args: &[Value]) -> Value {
@@ -133,20 +138,18 @@ pub fn memfrob(state: &mut State, args: &[Value]) -> Value {
     let addr = &args[0];
     let num = &args[1];
 
-    let x = Value::Concrete(0x2a, 0);
+    let x = vc(0x2a);
     let data = state.memory_read(&addr, &num);
     let mut new_data = vec![];
     for d in data {
         new_data.push(d.to_owned() ^ x.to_owned());
     }
-
     state.memory_write(addr, &new_data, &num);
-    //state.mem_copy(addr, data, num)
-    Value::Concrete(0, 0)
+    vc(0)
 }
 
 pub fn strlen(state: &mut State, args: &[Value]) -> Value {
-    state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0))
+    state.memory_strlen(&args[0], &vc(MAX_LEN))
 }
 
 pub fn strnlen(state: &mut State, args: &[Value]) -> Value {
@@ -154,101 +157,72 @@ pub fn strnlen(state: &mut State, args: &[Value]) -> Value {
 }
 
 pub fn gets(state: &mut State, args: &[Value]) -> Value {
-    syscall::read(
-        state,
-        &[
-            Value::Concrete(0, 0),
-            args[0].to_owned(),
-            Value::Concrete(1024, 0),
-        ],
-    ); // idk
-    args[0].to_owned()
+    readline(state, &[&[vc(0)], args].concat())
 }
 
 pub fn fgets(state: &mut State, args: &[Value]) -> Value {
     let fd = fileno(state, &args[2..3]);
-    syscall::read(
-        state,
-        &[
-            fd,
-            args[0].to_owned(),
-            args[1].to_owned() - Value::Concrete(1, 0),
-        ],
-    );
+    let buf = args[0].to_owned();
+    let len = args[1].to_owned() - vc(1);
+    syscall::read(state, &[fd, buf, len]);
     args[0].to_owned()
 }
 
 pub fn fputs(state: &mut State, args: &[Value]) -> Value {
     let fd = fileno(state, &args[1..2]);
     let length = strlen(state, &args[0..1]);
-    syscall::write(
-        state,
-        &[
-            fd,
-            args[0].to_owned(),
-            length
-        ],
-    );
-    Value::Concrete(0, 0)
+    syscall::write(state, &[fd, args[0].to_owned(), length]);
+    vc(0)
 }
 
 pub fn perror(state: &mut State, args: &[Value]) -> Value {
     let length = strlen(state, &args[0..1]);
-    syscall::write(
-        state,
-        &[
-            vc(2),
-            args[0].to_owned(),
-            length
-        ],
-    );
-    Value::Concrete(0, 0)
+    syscall::write(state, &[vc(2), args[0].to_owned(), length]);
+    vc(0)
 }
 
 pub fn fputc(state: &mut State, args: &[Value]) -> Value {
     let fd = fileno(state, &args[1..2]);
-    let fdn = state.solver.evalcon_to_u64(&fd).unwrap_or_default() as usize;
+    let fdn = state.solver.evalcon_to_u64(&fd).unwrap_or(1) as usize;
     state.filesystem.write(fdn, vec![args[0].to_owned()]);
-    Value::Concrete(0, 0)
+    vc(0)
 }
 
 pub fn feof(state: &mut State, args: &[Value]) -> Value {
     let fd = fileno(state, &args[1..2]);
-    let fdn = state.solver.evalcon_to_u64(&fd).unwrap_or_default() as usize;
+    let fdn = state.solver.evalcon_to_u64(&fd).unwrap_or(1) as usize;
     let f = &state.filesystem.files[fdn];
-    Value::Concrete(!(f.position == f.content.len()-1) as u64, 0)
+    vc(!(f.position == f.content.len() - 1) as u64)
 }
 
 pub fn strcpy(state: &mut State, args: &[Value]) -> Value {
-    let length =
-        state.memory_strlen(&args[1], &Value::Concrete(MAX_LEN, 0)) + Value::Concrete(1, 0);
+    let length = state.memory_strlen(&args[1], &vc(MAX_LEN)) + vc(1);
     state.memory_move(&args[0], &args[1], &length);
     args[0].to_owned()
 }
 
 pub fn stpcpy(state: &mut State, args: &[Value]) -> Value {
-    let length = state.memory_strlen(&args[1], &Value::Concrete(MAX_LEN, 0));
+    let length = state.memory_strlen(&args[1], &vc(MAX_LEN));
     strcpy(state, args) + length
 }
 
 pub fn strdup(state: &mut State, args: &[Value]) -> Value {
-    let length =
-        state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0)) + Value::Concrete(1, 0);
-    let new_addr = Value::Concrete(state.memory.alloc(&length), 0);
+    let length = state.memory_strlen(&args[0], &vc(MAX_LEN)) + vc(1);
+    let new_addr = vc(state.memory.alloc(&length));
     state.memory_move(&new_addr, &args[0], &length);
     new_addr
 }
 
+// what a weird function
 pub fn strdupa(state: &mut State, args: &[Value]) -> Value {
-    let length =
-        state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0)) + Value::Concrete(1, 0);
+    let length = state.memory_strlen(&args[0], &vc(MAX_LEN)) + vc(1);
     strdup(state, args) + length
 }
 
 // TODO for strn stuff I may need to add a null?
 pub fn strndup(state: &mut State, args: &[Value]) -> Value {
     let length = state.memory_strlen(&args[0], &args[1]);
-    let new_addr = Value::Concrete(state.memory.alloc(&length), 0);
+    let new_addr = vc(state.memory.alloc(&length));
     state.memory_move(&new_addr, &args[0], &length);
     new_addr
 }
@@ -269,16 +243,15 @@ pub fn strncpy(state: &mut State, args: &[Value]) -> Value {
 }
 
 pub fn strcat(state: &mut State, args: &[Value]) -> Value {
-    let length1 = state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0));
-    let length2 =
-        state.memory_strlen(&args[1], &Value::Concrete(MAX_LEN, 0)) + Value::Concrete(1, 0);
+    let length1 = state.memory_strlen(&args[0], &vc(MAX_LEN));
+    let length2 = state.memory_strlen(&args[1], &vc(MAX_LEN)) + vc(1);
     state.memory_move(&args[0].add(&length1), &args[1], &length2);
     args[0].to_owned()
 }
 
 pub fn strncat(state: &mut State, args: &[Value]) -> Value {
     let length1 = state.memory_strlen(&args[0], &args[2]);
-    let length2 = state.memory_strlen(&args[1], &args[2]) + Value::Concrete(1, 0);
+    let length2 = state.memory_strlen(&args[1], &args[2]) + vc(1);
     state.memory_move(&args[0].add(&length1), &args[1], &length2);
     args[0].to_owned()
 }
@@ -286,7 +259,6 @@ pub fn strncat(state: &mut State, args: &[Value]) -> Value {
 pub fn memset(state: &mut State, args: &[Value]) -> Value {
     let mut data = vec![];
     let length = state.solver.max_value(&args[2]);
-
     for _ in 0..length {
         data.push(args[1].to_owned());
     }
@@ -295,7 +267,7 @@ pub fn memset(state: &mut State, args: &[Value]) -> Value {
     args[0].to_owned()
 }
 
-pub fn memchr_helper(state: &mut State, args: &[Value], reverse: bool) -> Value {
+fn memchr_helper(state: &mut State, args: &[Value], reverse: bool) -> Value {
     state.memory_search(&args[0], &args[1], &args[2], reverse)
 }
 
@@ -307,13 +279,12 @@ pub fn memrchr(state: &mut State, args: &[Value]) -> Value {
     memchr_helper(state, args, true)
 }
 
-pub fn strchr_helper(state: &mut State, args: &[Value], reverse: bool) -> Value {
-    let length = state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0));
-    memchr_helper(
-        state,
-        &[args[0].to_owned(), args[1].to_owned(), length],
-        reverse,
-    )
+fn strchr_helper(state: &mut State, args: &[Value], reverse: bool) -> Value {
+    let length = state.memory_strlen(&args[0], &vc(MAX_LEN));
+    let string = args[0].to_owned();
+    let c = args[1].and(&vc(0xff));
+
+    memchr_helper(state, &[string, c, length], reverse)
 }
 
 pub fn strchr(state: &mut State, args: &[Value]) -> Value {
@@ -329,9 +300,9 @@ pub fn memcmp(state: &mut State, args: &[Value]) -> Value {
 }
 
 pub fn strcmp(state: &mut State, args: &[Value]) -> Value {
-    let len1 = state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0));
-    let len2 = state.memory_strlen(&args[1], &Value::Concrete(MAX_LEN, 0));
-    let length = state.solver.conditional(&(len1.ult(&len2)), &len1, &len2) + Value::Concrete(1, 0);
+    let len1 = state.memory_strlen(&args[0], &vc(MAX_LEN));
+    let len2 = state.memory_strlen(&args[1], &vc(MAX_LEN));
+    let length = state.cond(&(len1.ult(&len2)), &len1, &len2) + vc(1);
 
     state.memory_compare(&args[0], &args[1], &length)
 }
@@ -339,14 +310,14 @@ pub fn strcmp(state: &mut State, args: &[Value]) -> Value {
 pub fn strncmp(state: &mut State, args: &[Value]) -> Value {
     let len1 = state.memory_strlen(&args[0], &args[2]);
     let len2 = state.memory_strlen(&args[1], &args[2]);
-    let length = state.solver.conditional(&(len1.ult(&len2)), &len1, &len2) + Value::Concrete(1, 0);
+    let length = state.cond(&(len1.ult(&len2)), &len1, &len2) + vc(1);
 
     state.memory_compare(&args[0], &args[1], &length)
 }
 
 // TODO properly handle sym slens
 // idk if I will ever do this ^. it is super complicated
-// and the performance would likely be shit anyway
+// and the performance would likely be bad anyway
 pub fn memmem(state: &mut State, args: &[Value]) -> Value {
     let len = state.solver.evalcon_to_u64(&args[3]).unwrap() as usize;
     let mut needle_val = state.memory_read_value(&args[2], len);
@@ -356,16 +327,15 @@ pub fn memmem(state: &mut State, args: &[Value]) -> Value {
         state.solver.to_bv(&needle_val, 8 * len as u32),
         needle_val.get_taint(),
     );
-    memchr_helper(
-        state,
-        &[args[0].to_owned(), needle_val, args[1].to_owned()],
-        false,
-    )
+
+    let mem = args[0].to_owned();
+    let length = args[1].to_owned();
+    memchr_helper(state, &[mem, needle_val, length], false)
 }
 
 pub fn strstr(state: &mut State, args: &[Value]) -> Value {
-    let dlen = state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0));
-    let slen = state.memory_strlen(&args[1], &Value::Concrete(MAX_LEN, 0));
+    let dlen = state.memory_strlen(&args[0], &vc(MAX_LEN));
+    let slen = state.memory_strlen(&args[1], &vc(MAX_LEN));
     let len = state.solver.evalcon_to_u64(&slen).unwrap() as usize;
     let needle_val = state.memory_read_value(&args[0], len);
     memchr_helper(state, &[args[0].to_owned(), needle_val, dlen], false)
@@ -375,13 +345,17 @@ pub fn malloc(state: &mut State, args: &[Value]) -> Value {
     state.memory_alloc(&args[0])
 }
 
+pub fn realloc(state: &mut State, args: &[Value]) -> Value {
+    malloc(state, &args[1..])
+}
+
 pub fn calloc(state: &mut State, args: &[Value]) -> Value {
     state.memory_alloc(&args[0].mul(&args[1]))
 }
 
 pub fn free(state: &mut State, args: &[Value]) -> Value {
     state.memory_free(&args[0]);
-    Value::Concrete(0, 0)
+    vc(0)
 }
 
 pub fn mmap(state: &mut State, args: &[Value]) -> Value {
@@ -414,7 +388,7 @@ pub fn __libc_start_main(state: &mut State, args: &[Value]) -> Value {
     state.registers.set_with_alias("A1", argv);
 
     // TODO set env
-    state.registers.set_with_alias("A2", Value::Concrete(0, 0));
+    state.registers.set_with_alias("A2", vc(0));
 
     // uh in case we are overwriting A0
     args[1].to_owned()
@@ -458,7 +432,7 @@ type = struct _IO_FILE {
                          }
 */
 
-// beginning of shitty FILE function support
+// beginning of bad FILE function support
 
 // _fileno offset from above linux x86_64
 pub const LINUX_FILENO_OFFSET: u64 = 112;
@@ -468,27 +442,25 @@ pub const MACOS_FILENO_OFFSET: u64 = 18;
 
 pub fn fileno(state: &mut State, args: &[Value]) -> Value {
     let fd_addr = if state.info.bin.os == "darwin" {
-        args[0].add(&Value::Concrete(MACOS_FILENO_OFFSET, 0))
+        args[0].add(&vc(MACOS_FILENO_OFFSET))
     } else {
-        args[0].add(&Value::Concrete(LINUX_FILENO_OFFSET, 0))
+        args[0].add(&vc(LINUX_FILENO_OFFSET))
     };
-
     state.memory_read_value(&(fd_addr), 4)
 }
 
 pub fn fopen(state: &mut State, args: &[Value]) -> Value {
-    // we are reaching levels of shit code previously undreamt
+    // we are reaching levels of jank code previously undreamt
     let fd = syscall::open(state, args);
-    let file_struct = state.memory.alloc(&Value::Concrete(216, 0));
+    let file_struct = state.memory.alloc(&vc(216));
 
     let fd_addr = if state.info.bin.os == "darwin" {
-        vc(file_struct).add(&Value::Concrete(MACOS_FILENO_OFFSET, 0))
+        vc(file_struct).add(&vc(MACOS_FILENO_OFFSET))
     } else {
-        vc(file_struct).add(&Value::Concrete(LINUX_FILENO_OFFSET, 0))
+        vc(file_struct).add(&vc(LINUX_FILENO_OFFSET))
     };
-
     state.memory_write_value(&fd_addr, &fd, 4);
-    Value::Concrete(file_struct, 0)
+    vc(file_struct)
 }
 
 pub fn fclose(state: &mut State, args: &[Value]) -> Value {
@@ -517,153 +489,29 @@ pub fn ftell(state: &mut State, args: &[Value]) -> Value {
     vc(state.filesystem.files[fdn].position as u64)
 }
 
-/*
- * From SO
- * atoi reads digits from the buffer until it can't any more. It stops when it
- * encounters any character that isn't a digit, except whitespace (which it skips)
- * or a '+' or a '-' before it has seen any digits (which it uses to select the
- * appropriate sign for the result). It returns 0 if it saw no digits.
- */
-#[inline]
-fn vc(n: u64) -> Value {
-    Value::Concrete(n, 0)
-}
-
-// is digit
-#[inline]
-fn isdig(c: &Value) -> Value {
-    c.ult(&Value::Concrete(0x3a, 0)) & !c.ult(&Value::Concrete(0x30, 0))
-}
-
 // is whitespace
 fn _isws(c: &Value) -> Value {
-    c.eq(&Value::Concrete(0x09, 0))
-        | c.eq(&Value::Concrete(0x20, 0))
-        | c.eq(&Value::Concrete(0x0d, 0))
-        | c.eq(&Value::Concrete(0x0a, 0))
-}
-
-fn bv_pow(bv: &Value, exp: u32) -> Value {
-    let mut result = vc(1);
-    for _ in 0..exp {
-        result = result * bv.clone();
-    }
-    result
-}
-
-// is valid digit of base
-fn isbasedigit(state: &State, c: &Value, base: &Value) -> Value {
-    state.solver.conditional(
-        &base.ult(&vc(11)),
-        &(c.ult(&(vc('0' as u64) + base.clone())) & !c.ult(&vc('0' as u64))),
-        &(isdig(c)
-            | (c.ult(&(vc('a' as u64) + base.sub(&vc(10)))) & !c.ult(&vc('a' as u64)))
-            | (c.ult(&(vc('A' as u64) + base.sub(&vc(10)))) & !c.ult(&vc('A' as u64)))),
-    )
-}
-
-fn tonum(state: &State, c: &Value) -> Value {
-    let alpha = state.solver.conditional(
-        &c.ulte(&vc('Z' as u64)),
-        &c.sub(&vc('A' as u64 - 10)),
-        &c.sub(&vc('a' as u64 - 10)),
-    );
-
-    state
-        .solver
-        .conditional(&c.ulte(&vc('9' as u64)), &c.sub(&vc('0' as u64)), &alpha)
-}
-
-fn atoi_concrete(state: &mut State, addr: &Value, base: &Value, len: usize) -> Value {
-    let numstr = state.memory_read_string(addr.as_u64().unwrap(), len);
-    let numstr = numstr.trim_start(); // trim whitespace
-
-    let start = if &numstr[0..2] == "0x" { 2 } else { 0 }; // offset
-                                                           // oof this is rough, atoi / strtol stop at first nondigit, from_str_radix gives 0
-    let end = if let Some(n) = numstr[start + 1..]
-        .chars()
-        .position(|c| isbasedigit(state, &vc(c as u64), base).as_u64().unwrap() != 1)
-    {
-        start + n + 1
-    } else {
-        len
-    }; // oof
-
-    let numopt = u64::from_str_radix(&numstr[start..end], base.as_u64().unwrap() as u32);
-    numopt.map(vc).unwrap_or_default()
-}
-
-// for now and maybe forever this only works for strings that
-// don't have garbage in them. so only strings with digits or +/-
-pub fn atoi_helper(state: &mut State, addr: &Value, base: &Value, size: u64) -> Value {
-    let length = state.memory_strlen(&addr, &Value::Concrete(64, 0));
-    let data = state.memory_read(&addr, &length);
-    let len = data.len();
-
-    state.assert_value(&length.eq(&vc(len as u64)));
-    if len == 0 {
-        return Value::Concrete(0, 0);
-    }
-
-    // gonna take the easy way out and special case out all concrete
-    if addr.is_concrete() && base.is_concrete() && data.iter().all(|x| x.is_concrete()) {
-        return atoi_concrete(state, addr, base, len);
-    }
-
-    let mut result = Value::Concrete(0, 0);
-
-    // multiplier for negative nums
-    let neg_mul = state.solver.conditional(
-        &data[0].eq(&vc('-' as u64)),
-        &Value::Concrete(-1i64 as u64, 0),
-        &Value::Concrete(1, 0),
-    );
-
-    for (i, d) in data.iter().enumerate() {
-        let dx = d.uext(&vc(8));
-        let exp = (len - i - 1) as u32;
-
-        // digit or + / -
-        let cond = if i == 0 {
-            isbasedigit(state, &dx, base) | dx.eq(&vc('-' as u64)) | dx.eq(&vc('+' as u64))
-        } else {
-            isbasedigit(state, &dx, base)
-        };
-        state.assert_value(&cond);
-
-        // add d*10**n to result
-        result = result
-            + state.solver.conditional(
-                &!isbasedigit(state, &dx, base),
-                &vc(0),
-                &(bv_pow(base, exp) * tonum(state, &dx)),
-            );
-    }
-    // this assertion is much faster than slicing dx
-    let mask = (1i64.wrapping_shl(size as u32) - 1) as u64;
-    state.assert_value(&result.ulte(&Value::Concrete(mask, 0)));
-
-    result * neg_mul
+    c.eq(&vc(0x09)) | c.eq(&vc(0x20)) | c.eq(&vc(0x0d)) | c.eq(&vc(0x0a))
 }
 
 pub fn atoi(state: &mut State, args: &[Value]) -> Value {
-    atoi_helper(state, &args[0], &vc(10), 32)
+    format::atoi_helper(state, &args[0], &vc(10), 32)
 }
 
 pub fn atol(state: &mut State, args: &[Value]) -> Value {
     let bits = state.memory.bits;
-    atoi_helper(state, &args[0], &vc(10), bits)
+    format::atoi_helper(state, &args[0], &vc(10), bits)
 }
 
 pub fn atoll(state: &mut State, args: &[Value]) -> Value {
-    atoi_helper(state, &args[0], &vc(10), 64)
+    format::atoi_helper(state, &args[0], &vc(10), 64)
 }
 
 pub fn strtoll(state: &mut State, args: &[Value]) -> Value {
     // not perfect but idk
     if let Value::Concrete(addr, _) = args[1] {
         if addr != 0 {
-            let length = state.memory_strlen(&args[0], &Value::Concrete(64, 0));
+            let length = state.memory_strlen(&args[0], &vc(64));
             state.memory_write_value(
                 &args[1],
                 &args[0].add(&length),
@@ -672,7 +520,7 @@ pub fn strtoll(state: &mut State, args: &[Value]) -> Value {
         }
     }
 
-    atoi_helper(state, &args[0], &args[2], 64)
+    format::atoi_helper(state, &args[0], &args[2], 64)
 }
 
 pub fn strtod(state: &mut State, args: &[Value]) -> Value {
@@ -684,78 +532,18 @@ pub fn strtol(state: &mut State, args: &[Value]) -> Value {
     strtoll(state, args).slice(bits - 1, 0)
 }
 
-pub fn itoa_helper(
-    state: &mut State,
-    value: &Value,
-    string: &Value,
-    base: &Value,
-    sign: bool,
-    size: usize,
-) -> Value {
-    let mut data = vec![];
-
-    // condition to add a minus sign -
-    let neg_cond = &(value.slt(&vc(0)) & base.eq(&vc(10)) & vc(sign as u64));
-
-    let uval = state
-        .solver
-        .conditional(&neg_cond, &value.mul(&vc(-1i64 as u64)), &value);
-
-    let uval = Value::Symbolic(state.solver.to_bv(&uval, 128), 0);
-    let ubase = Value::Symbolic(state.solver.to_bv(&base, 128), 0);
-    let mut shift = Value::Symbolic(state.solver.bvv(0, 64), 0);
-
-    for i in 0..size as u32 {
-        let dx = uval.rem(&bv_pow(&ubase, i + 1)).div(&bv_pow(&ubase, i));
-
-        // shift that will be applied to remove 00000...
-        shift = state
-            .solver
-            .conditional(&!dx.clone(), &shift.add(&vc(8)), &vc(0));
-
-        data.push(state.solver.conditional(
-            &dx.ult(&vc(10)),
-            &dx.add(&vc('0' as u64)),
-            &dx.sub(&vc(10)).add(&vc('a' as u64)),
-        ));
-    }
-
-    data.reverse();
-
-    let bv = state.memory.pack(&data).as_bv().unwrap();
-    let shift_bits = 31 - bv.get_width().leading_zeros(); // log2(n)
-    let bv = bv.srl(&shift.as_bv().unwrap().slice(shift_bits - 1, 0));
-    let mut new_addr = string.clone();
-
-    if sign {
-        let b = state
-            .solver
-            .conditional(&neg_cond, &vc('-' as u64), &vc('+' as u64));
-
-        state.memory_write_value(&string, &b, 1);
-
-        // if we add a minus, write number to addr+1
-        new_addr = state
-            .solver
-            .conditional(&neg_cond, &(new_addr.clone() + vc(1)), &new_addr);
-    }
-    state.memory_write_value(&new_addr, &Value::Symbolic(bv, 0), data.len());
-
-    string.to_owned()
-}
-
 pub fn itoa(state: &mut State, args: &[Value]) -> Value {
-    itoa_helper(state, &args[0], &args[1], &args[2], true, 32)
+    format::itoa_helper(state, &args[0], &args[1], &args[2], true, 32)
 }
 
 pub fn islower(_state: &mut State, args: &[Value]) -> Value {
     let c = args[0].slice(7, 0);
-    c.ult(&Value::Concrete(0x7b, 0)) & !c.ult(&Value::Concrete(0x61, 0))
+    c.ult(&vc(0x7b)) & !c.ult(&vc(0x61))
 }
 
 pub fn isupper(_state: &mut State, args: &[Value]) -> Value {
     let c = args[0].slice(7, 0);
-    c.ult(&Value::Concrete(0x5b, 0)) & !c.ult(&Value::Concrete(0x41, 0))
+    c.ult(&vc(0x5b)) & !c.ult(&vc(0x41))
 }
 
 pub fn isalpha(state: &mut State, args: &[Value]) -> Value {
@@ -764,7 +552,7 @@ pub fn isalpha(state: &mut State, args: &[Value]) -> Value {
 
 pub fn isdigit(_state: &mut State, args: &[Value]) -> Value {
     let c = args[0].slice(7, 0);
-    c.ult(&Value::Concrete(0x3a, 0)) & !c.ult(&Value::Concrete(0x30, 0))
+    c.ult(&vc(0x3a)) & !c.ult(&vc(0x30))
 }
 
 pub fn isalnum(state: &mut State, args: &[Value]) -> Value {
@@ -773,52 +561,47 @@ pub fn isalnum(state: &mut State, args: &[Value]) -> Value {
 
 pub fn isblank(_state: &mut State, args: &[Value]) -> Value {
     let c = args[0].slice(7, 0);
-    c.eq(&Value::Concrete(0x20, 0)) | c.eq(&Value::Concrete(0x09, 0))
+    c.eq(&vc(0x20)) | c.eq(&vc(0x09))
 }
 
 pub fn iscntrl(_state: &mut State, args: &[Value]) -> Value {
     let c = args[0].slice(7, 0);
-    (c.ugte(&Value::Concrete(0, 0)) & c.ulte(&Value::Concrete(0x1f, 0)))
-        | c.eq(&Value::Concrete(0x7f, 0))
+    (c.ugte(&vc(0)) & c.ulte(&vc(0x1f))) | c.eq(&vc(0x7f))
 }
 
 pub fn toupper(state: &mut State, args: &[Value]) -> Value {
     let islo = islower(state, args);
-    state
-        .solver
-        .conditional(&islo, &args[0].sub(&vc(0x20)), &args[0])
+    state.cond(&islo, &args[0].sub(&vc(0x20)), &args[0])
 }
 
 pub fn tolower(state: &mut State, args: &[Value]) -> Value {
     let isup = isupper(state, args);
-    state
-        .solver
-        .conditional(&isup, &args[0].add(&vc(0x20)), &args[0])
+    state.cond(&isup, &args[0].add(&vc(0x20)), &args[0])
 }
 
 pub fn zero(_state: &mut State, _args: &[Value]) -> Value {
-    Value::Concrete(0, 0)
+    vc(0)
 }
 
 pub fn rand(state: &mut State, _args: &[Value]) -> Value {
-    let rand = state.symbolic_value(&format!("rand_{}", rand::thread_rng().gen::<u64>()), 64);
+    let r = rand::thread_rng().gen::<u64>();
+    let rand = state.symbolic_value(&format!("rand_{}", r), 64);
 
     let rand_vec = &mut state
         .context
         .entry("rand".to_string())
         .or_insert_with(Vec::new);
 
-    rand_vec.push(rand.to_owned());
-
+    rand_vec.push(rand.clone());
     rand
 }
 
 pub fn srand(_state: &mut State, _args: &[Value]) -> Value {
-    Value::Concrete(0, 0)
+    vc(0)
 }
 
 pub fn fflush(_state: &mut State, _args: &[Value]) -> Value {
-    Value::Concrete(0, 0)
+    vc(0)
 }
 
 pub fn getpid(state: &mut State, args: &[Value]) -> Value {
@@ -847,9 +630,9 @@ pub fn fork(state: &mut State, args: &[Value]) -> Value {
 
 pub fn brk(state: &mut State, args: &[Value]) -> Value {
     let addr = state.solver.evalcon_to_u64(&args[0]).unwrap();
-    let current = syscall::sbrk(state, &[vc(0)]);
-    let new = syscall::brk(state, args);
-    if current.as_u64().unwrap() == addr || new.as_u64().unwrap() != addr {
+    let current = syscall::sbrk(state, &[vc(0)]).as_u64();
+    let new = syscall::brk(state, args).as_u64();
+    if current.unwrap() == addr || new.unwrap() != addr {
         vc(0)
     } else {
         vc(-1i64 as u64)
@@ -861,19 +644,111 @@ pub fn sbrk(state: &mut State, args: &[Value]) -> Value {
 }
 
 pub fn getpagesize(_state: &mut State, _args: &[Value]) -> Value {
-    Value::Concrete(0x1000, 0)
+    vc(0x1000)
 }
 
 pub fn gethostname(state: &mut State, args: &[Value]) -> Value {
-    let addr = state.solver.evalcon_to_u64(&args[0]).unwrap();
+    let addr = state.solver.evalcon_to_u64(&args[0]).unwrap_or(0);
     state.memory_write_string(addr, "radius");
-    Value::Concrete(0, 0)
+    vc(0)
+}
+
+// hardcode these for now idk
+const OPTIND: u64 = 0xfffd0000;
+const OPTARG: u64 = 0xfffd0004;
+
+fn getopt_setup(state: &mut State) {
+    let relocs = state.r2api.get_relocations().unwrap_or_default();
+
+    let optind_addr = relocs
+        .iter()
+        .find(|r| r.name == "optind")
+        .map(|r| r.vaddr)
+        .unwrap_or(0);
+
+    let optarg_addr = relocs
+        .iter()
+        .find(|r| r.name == "optarg")
+        .map(|r| r.vaddr)
+        .unwrap_or(0);
+
+    let optind_ptr = state.memory_read_value(&vc(optind_addr), 4);
+
+    if let Some(ind) = optind_ptr.as_u64() {
+        if ind != OPTIND {
+            state.memory_write_ptr(&vc(optind_addr), &vc(OPTIND));
+            state.memory_write_ptr(&vc(optarg_addr), &vc(OPTARG));
+            state.memory_write_value(&vc(OPTIND), &vc(1), 4);
+            state.memory_write_ptr(&vc(OPTARG), &vc(0));
+        }
+    }
+}
+
+/*
+from the man pages
+
+"By default, getopt() permutes the contents of argv as it scans,
+so that eventually all the nonoptions are at the end."
+
+uhhhh i am not gonna do that for right now? also wat
+*/
+
+// this is not even close to being a faithful representation
+// of the actual (insane) semantics of getopt
+pub fn getopt(state: &mut State, args: &[Value]) -> Value {
+    getopt_setup(state);
+
+    let ptr = state.memory.bits / 8;
+    let optind_val = state.memory_read_value(&vc(OPTIND), 4);
+    let optind = state.solver.evalcon_to_u64(&optind_val).unwrap_or(1);
+    let argc = state.solver.evalcon_to_u64(&args[0]).unwrap_or(1);
+
+    if optind >= argc {
+        vc(-1i64 as u64)
+    } else {
+        let optstr_addr = state.solver.evalcon_to_u64(&args[2]).unwrap_or(0);
+        let optstr = state.memory_read_cstring(optstr_addr);
+
+        let argv_addr = state.memory_read_ptr(&args[1].add(&vc(optind * ptr)));
+        let argv_len = strlen(state, &[argv_addr.clone()]);
+
+        let arg = state.memory_read(&argv_addr, &argv_len);
+        let mut result = vc(-1i64 as u64);
+        if arg.len() < 2 {
+            return result;
+        }
+
+        let is_opt = arg[0].eq(&vc('-' as u64));
+        result = state.cond(&is_opt, &vc('?' as u64), &result);
+        let mut optarg = state.memory_read_ptr(&vc(OPTARG));
+
+        for index in 0..optstr.len() {
+            let c = vc(optstr.as_bytes()[index] as u64);
+            let is_c = arg[1].eq(&c);
+            result = state.cond(&is_opt.and(&is_c), &c, &result);
+
+            if arg.len() > 2 && index + 1 < optstr.len() && &optstr[index + 1..index + 2] == ":" {
+                // argument may be next argv, nvmd disallow
+                //let newarg_addr = args[1].add(&vc((optind+1)*ptr));
+
+                // if opt is c which has arg, arg must be in same argv (eg -xfoo)
+                let arg_cond = (!is_c.clone()).or(&is_c.and(&!arg[2].eq(&vc(0))));
+                state.assert(&arg_cond);
+
+                optarg = state.cond(&is_c, &argv_addr.add(&vc(2)), &optarg);
+                state.memory_write_ptr(&vc(OPTARG), &optarg);
+            }
+        }
+
+        state.memory_write_value(&vc(OPTIND), &vc(optind + 1), 4);
+        result
+    }
 }
 
 // fully symbolic getenv
 pub fn getenv(state: &mut State, args: &[Value]) -> Value {
     if state.context.get("env").is_none() {
-        return Value::Concrete(0, 0);
+        return vc(0);
     }
 
     let arg_ptr = args[0].to_owned();
@@ -882,32 +757,25 @@ pub fn getenv(state: &mut State, args: &[Value]) -> Value {
     let bits = state.memory.bits as usize;
     let mut env_ptr = state.context["env"][0].clone();
     let mut result = vc(0);
+    let eqs = vc('=' as u64);
     loop {
         let var_ptr = state.memory_read_value(&env_ptr, bits / 8);
 
         if state.solver.check_sat(&var_ptr.eq(&vc(0))) {
-            state.assert_value(&var_ptr.eq(&vc(0)));
+            state.assert(&var_ptr.eq(&vc(0)));
             break;
         }
 
         let full_length = state.memory_strlen(&var_ptr, &vc(MAX_LEN));
-        let name_end = state.memory_search(&var_ptr, &vc('=' as u64), &full_length, false);
+        let name_end = state.memory_search(&var_ptr, &eqs, &full_length, false);
 
-        let name_length =
-            state
-                .solver
-                .conditional(&name_end.eq(&vc(0)), &full_length, &name_end.sub(&var_ptr));
+        let name_length = state.cond(&name_end.eq(&vc(0)), &full_length, &name_end.sub(&var_ptr));
 
         let value_ptr = var_ptr.add(&name_length).add(&vc(1));
-        let long_len =
-            state
-                .solver
-                .conditional(&arg_length.ugte(&name_length), &arg_length, &name_length);
+        let long_len = state.cond(&arg_length.ugte(&name_length), &arg_length, &name_length);
 
         let cmp = state.memory_compare(&arg_ptr, &var_ptr, &long_len);
-        result = state
-            .solver
-            .conditional(&cmp.eq(&vc(0)), &value_ptr, &result);
+        result = state.cond(&cmp.eq(&vc(0)), &value_ptr, &result);
 
         env_ptr = env_ptr + vc(bits as u64 / 8);
     }
@@ -916,13 +784,13 @@ pub fn getenv(state: &mut State, args: &[Value]) -> Value {
 
 // the first arg is always the real path idk
 pub fn realpath(state: &mut State, args: &[Value]) -> Value {
-    let length = state.memory_strlen(&args[0], &Value::Concrete(MAX_LEN, 0));
+    let length = state.memory_strlen(&args[0], &vc(MAX_LEN));
     state.memory_move(&args[1], &args[0], &length);
     args[1].to_owned()
 }
 
 pub fn sleep(_state: &mut State, _args: &[Value]) -> Value {
-    Value::Concrete(0, 0)
+    vc(0)
 }
 
 pub fn open(state: &mut State, args: &[Value]) -> Value {
