@@ -1,6 +1,7 @@
 use crate::state::State;
 use crate::value::Value;
 
+pub mod format;
 pub mod fs;
 pub mod libc;
 pub mod syscall;
@@ -51,12 +52,16 @@ pub fn get_sims() -> Vec<Sim> {
         make_sim("fputs", libc::fputs, 2),
         make_sim("fputc", libc::fputc, 2),
         make_sim("feof", libc::feof, 1),
-        make_sim("fprintf", libc::fputs, 2), // fix
-        make_sim("printf", libc::printf, 1), // fix
-        make_sim("perror", libc::perror, 1), 
+        make_sim("fprintf", libc::fprintf, 2),
+        make_sim("printf", libc::printf, 1),
+        make_sim("sprintf", libc::sprintf, 1),
+        make_sim("vprintf", libc::printf, 1),   // fix
+        make_sim("vsprintf", libc::sprintf, 1), // fix
+        make_sim("perror", libc::perror, 1),
         make_sim("fflush", libc::fflush, 0),
-        make_sim("scanf", libc::scanf, 1), // fix
-        make_sim("__isoc99_scanf", libc::scanf, 1), // fix
+        make_sim("scanf", libc::scanf, 1),
+        make_sim("sscanf", libc::sscanf, 1),
+        make_sim("__isoc99_scanf", libc::scanf, 1),
         make_sim("fopen", libc::fopen, 2),
         make_sim("freopen", libc::fopen, 3),
         make_sim("fread", libc::fread, 4),
@@ -98,6 +103,7 @@ pub fn get_sims() -> Vec<Sim> {
         make_sim("bcopy", libc::bcopy, 3),
         make_sim("bzero", libc::bzero, 2),
         make_sim("malloc", libc::malloc, 1),
+        make_sim("realloc", libc::realloc, 1),
         make_sim("calloc", libc::calloc, 2),
         make_sim("free", libc::strnlen, 1),
         make_sim("mmap", libc::mmap, 4),
@@ -128,6 +134,8 @@ pub fn get_sims() -> Vec<Sim> {
         make_sim("stat", libc::stat, 2),
         make_sim("fstat", libc::fstat, 2),
         make_sim("lstat", libc::lstat, 2),
+        make_sim("getopt", libc::getopt, 3),
+        make_sim("getopt_long", libc::getopt, 4),
         make_sim("getuid", libc::getuid, 0),
         make_sim("getgid", libc::getgid, 0),
         make_sim("geteuid", libc::geteuid, 0),
@@ -145,6 +153,7 @@ pub fn get_sims() -> Vec<Sim> {
         make_sim("sysctl", zero, 1),
         make_sim("setbuf", zero, 2),
         make_sim("rand", libc::rand, 0),
+        make_sim("arc4random", libc::rand, 0),
         make_sim("srand", libc::srand, 1),
         make_sim("gethostname", libc::gethostname, 0),
         make_sim("getpagesize", libc::getpagesize, 0),
