@@ -3,8 +3,10 @@ use crate::r2_api::{Endian, Information, R2Api};
 use crate::registers::Registers;
 use crate::sims::fs::SimFilesytem;
 use crate::solver::{BitVec, Solver};
-use crate::value::Value;
+use crate::value::{vc, Value, byte_values};
 
+use std::cmp::Ordering;
+use std::rc::Rc;
 use std::u8;
 //use std::collections::HashMap;
 use ahash::AHashMap;
@@ -14,7 +16,7 @@ type HashMap<P, Q> = AHashMap<P, Q>;
 
 // event hooks could be a performance issue at some point
 // prolly not now cuz there are 10000 slower things
-pub const DO_EVENT_HOOKS: bool = true;
+pub const DO_EVENT_HOOKS: bool = false;
 
 #[derive(Debug, Clone, PartialEq, Hash, Eq)]
 pub enum EventTrigger {
@@ -55,7 +57,7 @@ pub enum EventContext {
     MoveContext(Value, Value, Value),
 }
 
-pub type EventHook = fn(&mut State, &EventContext);
+pub type EventHook = dyn Fn(&mut State, &EventContext);
 
 #[derive(Debug, Clone, PartialEq)]
 pub enum ExecMode {
@@ -98,7 +100,8 @@ pub enum StateStatus {
     PostMerge, // so we dont get caught in merge loop
     Unsat,
     Inactive,
-    Crash,
+    Crash(u64, char),
+    Exit,
 }
 
 #[derive(Clone)]
@@ -115,15 +118,39 @@ pub struct State {
     pub status: StateStatus,
     pub context: HashMap<String, Vec<Value>>,
     pub taints: HashMap<String, u64>,
-    pub hooks: HashMap<Event, EventHook>,
+    pub hooks: HashMap<Event, Rc<EventHook>>,
+    pub visits: HashMap<u64, usize>,
     pub pid: u64,
     pub backtrace: Vec<(u64, u64)>,
     pub blank: bool,
     pub debug: bool,
+    pub check: bool,
     pub strict: bool,
     pub has_event_hooks: bool,
 }
 
+// state equality is based on PC visit count
+// in order to use a priority queue to manage states
+impl PartialEq for State {
+    fn eq(&self, other: &Self) -> bool {
+        other.get_visit() == self.get_visit()
+    }
+}
+
+impl Eq for State {}
+
+impl Ord for State {
+    fn cmp(&self, other: &Self) -> Ordering {
+        other.get_visit().cmp(&self.get_visit())
+    }
+}
+
+impl PartialOrd for State {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
 impl State {
     /// Create a new state, should generally not be called directly
     pub fn new(
@@ -147,7 +174,7 @@ impl State {
 
         let solver = Solver::new(eval_max);
         let registers = Registers::new(r2api, solver.clone(), blank);
-        let memory = Memory::new(r2api, solver.clone(), blank, check);
+        let memory = Memory::new(r2api, solver.clone(), blank);
 
         State {
             solver,
@@ -163,10 +190,12 @@ impl State {
             context: HashMap::new(),
             taints: HashMap::new(),
             hooks: HashMap::new(),
+            visits: HashMap::with_capacity(512),
             backtrace: Vec::with_capacity(128),
             pid: 1337, // sup3rh4x0r
             blank,
             debug,
+            check,
             strict,
             has_event_hooks: false,
         }
@@ -208,8 +237,8 @@ impl State {
 
         let esil_state = EsilState {
             mode: ExecMode::Uncon,
-            previous: Value::Concrete(0, 0),
-            current: Value::Concrete(0, 0),
+            previous: vc(0),
+            current: vc(0),
             last_sz: 64,
             stored_address: None,
             temp1: Vec::with_capacity(128),
@@ -231,24 +260,28 @@ impl State {
             context,
             taints: self.taints.clone(),
             hooks: self.hooks.clone(),
+            visits: self.visits.clone(),
             backtrace: self.backtrace.clone(),
             pid: self.pid,
             blank: self.blank,
             debug: self.debug,
+            check: self.check,
             strict: self.strict,
             has_event_hooks: self.has_event_hooks,
         }
     }
 
-    pub fn hook_event(&mut self, event: Event, hook: EventHook) {
+    pub fn hook_event(&mut self, event: Event, hook: Rc<EventHook>) {
         self.has_event_hooks = true;
         self.hooks.insert(event, hook);
     }
 
     pub fn do_hooked(&mut self, event: &Event, event_context: &EventContext) {
-        if let Some(hook) = self.hooks.get(event) {
-            hook(self, event_context)
+        if !self.hooks.contains_key(event) {
+            return;
         }
+        let hook = self.hooks.get(event).unwrap().clone();
+        hook(self, event_context)
     }
 
     /// Allocate a block of memory `length` bytes in size
@@ -287,6 +320,10 @@ impl State {
             self.do_hooked(&event, &EventContext::FreeContext(addr.to_owned()));
         }
 
+        if self.check && self.check_crash(addr, &vc(1), 'r') {
+            return vc(-1i64 as u64);
+        }
+
         let ret = self.memory.free_sym(addr, &mut self.solver);
 
         if DO_EVENT_HOOKS && self.has_event_hooks {
@@ -311,6 +348,10 @@ impl State {
             );
         }
 
+        if self.check && self.check_crash(address, length, 'r') {
+            return vec![];
+        }
+
         let ret = self.memory.read_sym_len(address, length, &mut self.solver);
 
         if DO_EVENT_HOOKS && self.has_event_hooks && (address.is_symbolic() || length.is_symbolic())
@@ -334,6 +375,9 @@ impl State {
             );
         }
 
+        if self.check && self.check_crash(address, length, 'r') {
+            return;
+        }
         let ret = self
             .memory
             .write_sym_len(address, values, length, &mut self.solver);
@@ -359,6 +403,10 @@ impl State {
             );
         }
 
+        if self.check && self.check_crash(address, &vc(length as u64), 'r') {
+            return vc(-1i64 as u64);
+        }
+
         let ret = self.memory.read_sym(address, length, &mut self.solver);
 
         if DO_EVENT_HOOKS && self.has_event_hooks && address.is_symbolic() {
@@ -381,6 +429,10 @@ impl State {
             );
         }
 
+        if self.check && self.check_crash(address, &vc(length as u64), 'w') {
+            return;
+        }
+
         let ret = self
             .memory
             .write_sym(address, value, length, &mut self.solver);
@@ -416,6 +468,10 @@ impl State {
             );
         }
 
+        if self.check && self.check_crash(addr, &vc(1), 'r') {
+            return vc(-1i64 as u64);
+        }
+
         let ret = self
             .memory
             .search(addr, needle, length, reverse, &mut self.solver);
@@ -450,6 +506,11 @@ impl State {
             );
         }
 
+        if self.check && (self.check_crash(src, &vc(1), 'r') || self.check_crash(dst, &vc(1), 'r'))
+        {
+            return vc(-1i64 as u64);
+        }
+
         let ret = self.memory.compare(dst, src, length, &mut self.solver);
 
         if DO_EVENT_HOOKS && self.has_event_hooks {
@@ -481,6 +542,11 @@ impl State {
             );
         }
 
+        // eh don't use the length here
+        if self.check && self.check_crash(addr, &vc(1), 'r') {
+            return vc(-1i64 as u64);
+        }
+
         let ret = self.memory.strlen(addr, length, &mut self.solver);
 
         if DO_EVENT_HOOKS && self.has_event_hooks {
@@ -512,6 +578,11 @@ impl State {
             );
         }
 
+        if self.check && (self.check_crash(src, length, 'r') || self.check_crash(dst, length, 'w'))
+        {
+            return;
+        }
+
         self.memory.memmove(dst, src, length, &mut self.solver);
 
         if DO_EVENT_HOOKS && self.has_event_hooks {
@@ -527,6 +598,18 @@ impl State {
         }
     }
 
+    /// Read pointer from `address`
+    pub fn memory_read_ptr(&mut self, address: &Value) -> Value {
+        let ptr_len = self.memory.bits as usize / 8;
+        self.memory_read_value(address, ptr_len)
+    }
+
+    /// Read pointer from `address`
+    pub fn memory_write_ptr(&mut self, address: &Value, value: &Value) {
+        let ptr_len = self.memory.bits as usize / 8;
+        self.memory_write_value(address, value, ptr_len)
+    }
+
     /// Read `length` bytes from `address`
     pub fn memory_read_bytes(&mut self, address: u64, length: usize) -> Vec<u8> {
         self.memory.read_bytes(address, length, &mut self.solver)
@@ -537,6 +620,14 @@ impl State {
         self.memory.read_string(address, length, &mut self.solver)
     }
 
+    /// Read a concrete c string from `address`
+    pub fn memory_read_cstring(&mut self, address: u64) -> String {
+        let length = self.memory_strlen(&vc(address), &vc(4096));
+        let len = self.solver.evalcon_to_u64(&length).unwrap_or(0);
+        self.memory
+            .read_string(address, len as usize, &mut self.solver)
+    }
+
     // this doesnt need to be here, just for consistency sake
     /// Write `string` to `address`
     pub fn memory_write_string(&mut self, address: u64, string: &str) {
@@ -558,26 +649,34 @@ impl State {
     }
 
     pub fn fill_file_string(&mut self, fd: usize, string: &str) {
-        let data: Vec<Value> = string
-            .chars()
-            .map(|c| Value::Concrete(c as u64, 0))
-            .collect();
-
+        let data = byte_values(string);
         self.filesystem.fill(fd, &data)
     }
 
+    pub fn dump_path(&mut self, path: &str) -> Vec<Value> {
+        if let Some(fd) = self.filesystem.getfd(path) {
+            self.filesystem.dump(fd)
+        } else {
+            vec![]
+        }
+    }
+
     pub fn dump_file(&mut self, fd: usize) -> Vec<Value> {
         self.filesystem.dump(fd)
     }
 
-    pub fn dump_file_string(&mut self, fd: usize) -> Option<String> {
+    pub fn dump_file_bytes(&mut self, fd: usize) -> Vec<u8> {
         let values = self.filesystem.dump(fd);
         let bytes = values
             .iter()
             .map(|v| self.solver.evalcon_to_u64(v).unwrap_or(0) as u8)
             .collect();
 
-        String::from_utf8(bytes).ok()
+        bytes
+    }
+
+    pub fn dump_file_string(&mut self, fd: usize) -> Option<String> {
+        String::from_utf8(self.dump_file_bytes(fd)).ok()
     }
 
     /// Apply this state to the radare2 instance. This writes all the values
@@ -603,11 +702,9 @@ impl State {
             let b = self.solver.evalcon_to_u64(&bval).unwrap() as u8;
             chunk.push(b);
 
-            if chunk.len() > 255 
-                || i >= addrs.len()-1
-                || addrs[i+1] != addrs[i]+1 
-            {
-                self.r2api.write(1+addr-chunk.len() as u64, chunk.clone());
+            if chunk.len() > 255 || i >= addrs.len() - 1 || addrs[i + 1] != addrs[i] + 1 {
+                self.r2api
+                    .write(1 + addr - chunk.len() as u64, chunk.clone());
                 chunk.clear();
             }
         }
@@ -705,10 +802,10 @@ impl State {
 
     /// constrain bytes of bitvector to be an exact string eg. "ABC"
     /// or use "[...]" to match a simple pattern eg. "[XYZa-z0-9]"
-    pub fn constrain_bytes(&mut self, bv: &BitVec, pattern: &str) {
+    pub fn constrain_bytes_bv(&mut self, bv: &BitVec, pattern: &str) {
         if &pattern[..1] != "[" {
             for (i, c) in pattern.chars().enumerate() {
-                self.assert(
+                self.assert_bv(
                     &bv.slice(8 * (i as u32 + 1) - 1, 8 * i as u32)
                         ._eq(&self.bvv(c as u64, 8)),
                 );
@@ -735,16 +832,34 @@ impl State {
                     }
                 }
 
-                self.assert(&self.solver.or_all(&assertions));
+                self.assert_bv(&self.solver.or_all(&assertions));
             }
         }
     }
 
     /// constrain bytes of bitvector to be an exact string eg. "ABC"
     /// or use "[...]" to match a simple pattern eg. "[XYZa-z0-9]"
-    pub fn constrain_bytes_value(&mut self, bv: &Value, pattern: &str) {
+    pub fn constrain_bytes(&mut self, bv: &Value, pattern: &str) {
         if let Value::Symbolic(s, _) = bv {
-            self.constrain_bytes(&s, pattern)
+            self.constrain_bytes_bv(&s, pattern)
+        }
+    }
+
+    /// constrain bytes of file with file descriptor
+    pub fn constrain_fd(&mut self, fd: usize, content: &str) {
+        let fbytes = self.dump_file(fd);
+        let cbytes = byte_values(content);
+        for i in 0..cbytes.len() {
+            if i < fbytes.len() {
+                self.assert(&cbytes[i].eq(&fbytes[i]));
+            }
+        }
+    }
+
+    /// constrain bytes of file 
+    pub fn constrain_file(&mut self, path: &str, content: &str) {
+        if let Some(fd) = self.filesystem.getfd(path) {
+            self.constrain_fd(fd, content);
         }
     }
 
@@ -758,6 +873,22 @@ impl State {
         }
     }
 
+    /// increment visit counter
+    pub fn visit(&mut self) {
+        if let Some(pc) = self.registers.get_pc().as_u64() {
+            self.visits.entry(pc).and_modify(|c| *c += 1).or_insert(1);
+        }
+    }
+
+    /// get visit counter
+    pub fn get_visit(&self) -> usize {
+        if let Some(pc) = self.registers.get_pc().as_u64() {
+            *self.visits.get(&pc).unwrap_or(&0)
+        } else {
+            0
+        }
+    }
+
     /// set status of state (active, inactive, merge, unsat...)
     pub fn set_status(&mut self, status: StateStatus) {
         self.status = status;
@@ -773,6 +904,36 @@ impl State {
         self.set_status(StateStatus::Inactive);
     }
 
+    /// convenience method to mark state crashed
+    pub fn set_crash(&mut self, addr: u64, perm: char) {
+        self.set_status(StateStatus::Crash(addr, perm));
+    }
+
+    pub fn check_crash(&mut self, addr: &Value, len: &Value, perm: char) -> bool {
+        let length = self.solver.max_value(len);
+        match addr {
+            Value::Concrete(address, _t) => {
+                let crash = !self.memory.check_permission(*address, length, perm);
+                if crash {
+                    self.set_crash(*address, perm);
+                }
+                crash
+            }
+            Value::Symbolic(address, _t) => {
+                let min = self.solver.min(address);
+                let max = self.solver.max(address);
+                let min_crash = !self.memory.check_permission(min, length, perm);
+                let max_crash = !self.memory.check_permission(max, length, perm);
+                if min_crash {
+                    self.set_crash(min, perm);
+                } else if max_crash {
+                    self.set_crash(max, perm);
+                }
+                min_crash || max_crash
+            }
+        }
+    }
+
     /// convenience method to break
     pub fn set_break(&mut self) {
         self.set_status(StateStatus::Break);
@@ -831,18 +992,23 @@ impl State {
     }
 
     /// Assert the truth of the given bitvector (value != 0)
-    pub fn assert(&mut self, bv: &BitVec) {
-        self.solver.assert(bv)
+    pub fn assert_bv(&mut self, bv: &BitVec) {
+        self.solver.assert_bv(bv)
     }
 
-    /// Check the truth of the given value
+    /// Assert the truth of the given `Value` (value != 0)
+    pub fn assert(&mut self, value: &Value) {
+        self.solver.assert(value)
+    }
+
+    /// Check the satisfiability of the given value
     pub fn check(&mut self, val: &Value) -> bool {
         self.solver.check_sat(val)
     }
 
-    /// Assert the truth of the given `Value` (lsb of value != 0)
-    pub fn assert_value(&mut self, value: &Value) {
-        self.solver.assert_value(value)
+    /// Get a conditional value
+    pub fn cond(&self, condition: &Value, if_val: &Value, else_val: &Value) -> Value {
+        self.solver.conditional(condition, if_val, else_val)
     }
 
     /// Evaluate multiple solutions to bv
@@ -851,7 +1017,7 @@ impl State {
     }
 
     /// Evaluate bytes from bitvector `bv`
-    pub fn evaluate_bytes(&mut self, bv: &BitVec) -> Option<Vec<u8>> {
+    pub fn evaluate_bytes_bv(&mut self, bv: &BitVec) -> Option<Vec<u8>> {
         let new_bv = bv; //self.translate(bv).unwrap();
         let mut data: Vec<u8> = vec![];
         if self.solver.is_sat() {
@@ -875,8 +1041,8 @@ impl State {
     }
 
     /// Evaluate bytes from bitvector `bv`
-    pub fn evaluate_string(&mut self, bv: &BitVec) -> Option<String> {
-        if let Some(bytes) = self.evaluate_bytes(bv) {
+    pub fn evaluate_string_bv(&mut self, bv: &BitVec) -> Option<String> {
+        if let Some(bytes) = self.evaluate_bytes_bv(bv) {
             String::from_utf8(bytes).ok()
         } else {
             None
@@ -884,12 +1050,12 @@ impl State {
     }
 
     /// Evaluate bytes from value
-    pub fn evaluate_bytes_value(&mut self, value: &Value) -> Option<Vec<u8>> {
-        self.evaluate_bytes(value.as_bv().as_ref().unwrap())
+    pub fn evaluate_bytes(&mut self, value: &Value) -> Option<Vec<u8>> {
+        self.evaluate_bytes_bv(value.as_bv().as_ref().unwrap())
     }
 
     /// Evaluate string from value
-    pub fn evaluate_string_value(&mut self, value: &Value) -> Option<String> {
-        self.evaluate_string(value.as_bv().as_ref().unwrap())
+    pub fn evaluate_string(&mut self, value: &Value) -> Option<String> {
+        self.evaluate_string_bv(value.as_bv().as_ref().unwrap())
     }
 }
